@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -38,9 +39,12 @@
 
 #include <directfb.h>
 
+#include <direct/messages.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
+#include <core/screens.h>
 #include <core/state.h>
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
@@ -100,6 +104,14 @@ static void ati128EngineSync( void *drv, void *dev )
      ati128_waitidle( adrv, adev );
 }
 
+static bool ati128_check_blend( CardState *state )
+{
+   if (state->dst_blend == DSBF_SRCALPHASAT)
+      return false;
+
+   return true;
+}
+
 static void ati128CheckState( void *drv, void *dev,
                               CardState *state, DFBAccelerationMask accel )
 {
@@ -122,8 +134,13 @@ static void ati128CheckState( void *drv, void *dev,
 
      /* if there are no other drawing flags than the supported */
      if (!(accel & ~ATI128_SUPPORTED_DRAWINGFUNCTIONS) &&
-         !(state->drawingflags & ~ATI128_SUPPORTED_DRAWINGFLAGS))
+         !(state->drawingflags & ~ATI128_SUPPORTED_DRAWINGFLAGS)) {
+          if (state->drawingflags & DSDRAW_BLEND &&
+              !ati128_check_blend( state ))
+               return;
+
           state->accel |= ATI128_SUPPORTED_DRAWINGFUNCTIONS;
+     }
 
      /* if there are no other blitting flags than the supported
         and the source has the minimum size */
@@ -133,6 +150,10 @@ static void ati128CheckState( void *drv, void *dev,
          state->source->width  >= 8 &&
          state->source->height >= 8 )
      {
+          if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL &&
+              !ati128_check_blend( state ))
+               return;
+
           switch (state->source->format) {
                case DSPF_RGB332:
                case DSPF_ARGB1555:
@@ -154,7 +175,7 @@ static void ati128SetState( void *drv, void *dev,
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     
+
      if (state->modified & SMF_SOURCE)
          adev->v_source = 0;
 
@@ -189,7 +210,7 @@ static void ati128SetState( void *drv, void *dev,
                }
           case DFXL_DRAWLINE:
                ati128_set_color( adrv, adev, state );
-               state->set |= DFXL_FILLRECTANGLE | DFXL_DRAWLINE | DFXL_DRAWRECTANGLE ;               
+               state->set |= DFXL_FILLRECTANGLE | DFXL_DRAWLINE | DFXL_DRAWRECTANGLE ;
                break;
 
           case DFXL_BLIT:
@@ -204,7 +225,7 @@ static void ati128SetState( void *drv, void *dev,
                break;
 
           default:
-               BUG( "unexpected drawing/blitting function" );
+               D_BUG( "unexpected drawing/blitting function" );
                break;
      }
 
@@ -412,7 +433,7 @@ static bool ati128DrawLine( void *drv, void *dev, DFBRegion *line )
      /* set the destination datatype */
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | BRUSH_SOLIDCOLOR | ROP3_SRCCOPY );
      ati128_out32( mmio, DP_MIX, ROP3_PATCOPY );
-     
+
      /* set start coorinates */
      ati128_out32( mmio, DST_Y_X, (S14(line->y1) << 16) | S12(line->x1));
      /* allow setting of last pel bit and polygon
@@ -527,17 +548,17 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                     ati128_out32( mmio, TEX_CNTL, 0 );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                return false;
      }
 
      ati128_out32( mmio, DP_DATATYPE, adev->ATI_dst_bpp | SRC_DSTCOLOR );
-     
+
      /* set the blend function */
      if (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
           ati128_out32( mmio, SCALE_3D_CNTL, adev->ATI_blend_function );
      else
-          ati128_out32( mmio, SCALE_3D_CNTL, SCALE_3D_CNTL_SCALE_3D_FN_SCALE ); 
+          ati128_out32( mmio, SCALE_3D_CNTL, SCALE_3D_CNTL_SCALE_3D_FN_SCALE );
 
      /* set up source data and copy type */
      ati128_out32( mmio, DP_MIX, ROP3_SRCCOPY | DP_SRC_RECT );
@@ -687,8 +708,9 @@ driver_init_driver( GraphicsDevice      *device,
      funcs->StretchBlit   = ati128StretchBlit;
 
      /* overlay support */
-     dfb_layers_register( device, driver_data, &atiOverlayFuncs );
-     
+     dfb_layers_register( dfb_screens_at(DSCID_PRIMARY),
+                          driver_data, &atiOverlayFuncs );
+
      return DFB_OK;
 }
 
@@ -758,25 +780,25 @@ driver_close_device( GraphicsDevice *device,
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
      volatile __u8    *mmio = adrv->mmio_base;
 
-     DEBUGMSG( "DirectFB/ATI128: FIFO Performance Monitoring:\n" );
-     DEBUGMSG( "DirectFB/ATI128:  %9d ati128_waitfifo calls\n",
+     D_DEBUG( "DirectFB/ATI128: FIFO Performance Monitoring:\n" );
+     D_DEBUG( "DirectFB/ATI128:  %9d ati128_waitfifo calls\n",
                adev->waitfifo_calls );
-     DEBUGMSG( "DirectFB/ATI128:  %9d register writes (ati128_waitfifo sum)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d register writes (ati128_waitfifo sum)\n",
                adev->waitfifo_sum );
-     DEBUGMSG( "DirectFB/ATI128:  %9d FIFO wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d FIFO wait cycles (depends on CPU)\n",
                adev->fifo_waitcycles );
-     DEBUGMSG( "DirectFB/ATI128:  %9d IDLE wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d IDLE wait cycles (depends on CPU)\n",
                adev->idle_waitcycles );
-     DEBUGMSG( "DirectFB/ATI128:  %9d FIFO space cache hits(depends on CPU)\n",
+     D_DEBUG( "DirectFB/ATI128:  %9d FIFO space cache hits(depends on CPU)\n",
                adev->fifo_cache_hits );
-     DEBUGMSG( "DirectFB/ATI128: Conclusion:\n" );
-     DEBUGMSG( "DirectFB/ATI128:  Average register writes/ati128_waitfifo"
+     D_DEBUG( "DirectFB/ATI128: Conclusion:\n" );
+     D_DEBUG( "DirectFB/ATI128:  Average register writes/ati128_waitfifo"
                "call:%.2f\n",
                adev->waitfifo_sum/(float)(adev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/ATI128:  Average wait cycles/ati128_waitfifo call:"
+     D_DEBUG( "DirectFB/ATI128:  Average wait cycles/ati128_waitfifo call:"
                " %.2f\n",
                adev->fifo_waitcycles/(float)(adev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/ATI128:  Average fifo space cache hits: %02d%%\n",
+     D_DEBUG( "DirectFB/ATI128:  Average fifo space cache hits: %02d%%\n",
                (int)(100 * adev->fifo_cache_hits/
                (float)(adev->waitfifo_calls)) );
 
