@@ -1,5 +1,5 @@
 /*
-   i810_overlay.c -- Video Overlay Support (based partly from 
+   i810_overlay.c -- Video Overlay Support (based partly from
                      XFree86 i810_video.c)
 
    (c) Copyright 2000-2002  convergence integrated media GmbH.
@@ -28,9 +28,12 @@
 #include <core/coredefs.h>
 #include <core/layers.h>
 #include <core/surfaces.h>
-#include <core/fbdev/fbdev.h>
+#include <core/screens.h>
+#include <core/screen.h>
 
-#include <misc/mem.h>
+#include <fbdev/fbdev.h>
+
+#include <direct/mem.h>
 
 #include <gfx/convert.h>
 
@@ -70,8 +73,7 @@ struct i810_ovl_regs {
 };
 
 typedef struct {
-	DFBRectangle          dest;
-	DFBDisplayLayerConfig config;
+	CoreLayerRegionConfig config;
 	int                   planar_bug;
 	struct i810_ovl_regs  *regs;
 } I810OverlayLayerData;
@@ -129,19 +131,20 @@ typedef struct {
 #define SRC_CONSTANT_ALPHA_BLEND  1 << 31
 #define MINUV_SCALE	        0x1
 
-#define I810FB_IOC_UPDATEOVERLAY       _IOW ('F', 0xF7, struct i810_ovl_regs)  
+#define I810FB_IOC_UPDATEOVERLAY       _IOW ('F', 0xF7, struct i810_ovl_regs)
 #define I810FB_IOC_UPDATEOVERLAYCMD    _IOW ('F', 0xF6, int)
 
-extern u32 i810_wait_for_space(I810DriverData *i810drv, 
+extern u32 i810_wait_for_space(I810DriverData *i810drv,
 			       I810DeviceData *i810dev,
 			       u32             space   );
 
 #define I810_OVERLAY_SUPPORTED_OPTIONS (DLOP_DST_COLORKEY | DLOP_DEINTERLACING)
 
-static void ovl_calc_regs (I810DriverData        *i810drv, 
+static void ovl_calc_regs (I810DriverData        *i810drv,
 			   I810OverlayLayerData  *i810ovl,
-			   DisplayLayer          *layer,
-			   DFBDisplayLayerConfig *config  );
+			   CoreLayer             *layer,
+			   CoreSurface           *surface,
+			   CoreLayerRegionConfig *config );
 
 static void update_overlay(I810DriverData       *i810drv,
 			   I810OverlayLayerData *i810ovl )
@@ -149,6 +152,18 @@ static void update_overlay(I810DriverData       *i810drv,
 	i810_writel(i810drv->mmio_base, OV0ADDR, i810drv->ovl_mem.physical);
 }
 	
+static void
+ovlOnOff( I810DriverData       *i810drv,
+	  I810OverlayLayerData *i810ovl,
+	  int                   on )
+{
+	if (on)
+		i810ovl->regs->ov0cmd |= 1;
+	else
+		i810ovl->regs->ov0cmd &= ~1;
+	update_overlay(i810drv, i810ovl);
+}
+
 static int
 ovlLayerDataSize()
 {
@@ -156,13 +171,13 @@ ovlLayerDataSize()
 }
 
 static DFBResult
-ovlInitLayer( GraphicsDevice             *device,
-              DisplayLayer               *layer,
-              DisplayLayerInfo           *layer_info,
-              DFBDisplayLayerConfig      *default_config,
-              DFBColorAdjustment         *default_adj,
-              void                       *driver_data,
-              void                       *layer_data )
+ovlInitLayer(
+              CoreLayer                  *layer,
+	      void                       *driver_data,
+	      void                       *layer_data,
+              DFBDisplayLayerDescription *description,
+              DFBDisplayLayerConfig      *config,
+              DFBColorAdjustment         *adjustment )
 {
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
 	I810DriverData *i810drv = (I810DriverData *) driver_data;
@@ -170,41 +185,38 @@ ovlInitLayer( GraphicsDevice             *device,
 	i810ovl->regs = (struct i810_ovl_regs *) i810drv->ovl_base;
 
 	/* set_capabilities */
-	layer_info->desc.caps = DLCAPS_SURFACE | DLCAPS_SCREEN_LOCATION |
+	description->caps = DLCAPS_SURFACE | DLCAPS_SCREEN_LOCATION |
 		DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST | DLCAPS_SATURATION |
-		DLCAPS_DST_COLORKEY | DLCAPS_OPACITY | DLCAPS_DEINTERLACING; 
+		DLCAPS_DST_COLORKEY | DLCAPS_OPACITY | DLCAPS_DEINTERLACING;
 
-	layer_info->desc.type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
+	description->type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
 	/* set name */
-	snprintf( layer_info->desc.name,
+	snprintf( description->name,
 		  DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "Intel 810/815 Overlay" );
 
 	/* fill out the default configuration */
-	default_config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
+	config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
 		DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS;
-	default_config->width       = 640;
-	default_config->height      = 480;
-	default_config->pixelformat = DSPF_YUY2;
-	default_config->buffermode  = DLBM_FRONTONLY;
-	default_config->options     = DLOP_NONE;
+	config->width       = 640;
+	config->height      = 480;
+	config->pixelformat = DSPF_YUY2;
+	config->buffermode  = DLBM_FRONTONLY;
+	config->options     = DLOP_NONE;
 
 	/* fill out default color adjustment,
 	   only fields set in flags will be accepted from applications */
-	default_adj->flags      = DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_SATURATION;
-	default_adj->brightness = 0x8000;
-	default_adj->contrast   = 0x8000;
-	default_adj->saturation = 0x8000;
-     
-	/* initialize destination rectangle */
-	dfb_primary_layer_rectangle( 0.0f, 0.0f, 1.0f, 1.0f, &i810ovl->dest );
-     
+	adjustment->flags      = DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_SATURATION;
+	adjustment->brightness = 0x8000;
+	adjustment->contrast   = 0x8000;
+	adjustment->saturation = 0x8000;
+
 	i810ovl->regs->yrgb_vph  = 0;
 	i810ovl->regs->uv_vph    = 0;
 	i810ovl->regs->horz_ph   = 0;
 	i810ovl->regs->init_ph   = 0;
 	i810ovl->regs->dwinpos   = 0;
-	i810ovl->regs->dwinsz    = (640 << 16) | 480; 
-	i810ovl->regs->swid      = 640 | (640 << 15);       
+	i810ovl->regs->dwinsz    = (640 << 16) | 480;
+	i810ovl->regs->swid      = 640 | (640 << 15);
 	i810ovl->regs->swidqw    = (640 >> 3) | (640 << 12);
 	i810ovl->regs->sheight   = 480 | (480 << 15);
 	i810ovl->regs->yrgbscale = 0x80004000; /* scale factor 1 */
@@ -215,18 +227,18 @@ ovlInitLayer( GraphicsDevice             *device,
 	i810ovl->regs->sclrkvh = 0;
 	i810ovl->regs->sclrkvl = 0;
 	i810ovl->regs->sclrkm  = 0; /* source color key disable */
-	i810ovl->regs->ov0conf = 0; /* two 720 pixel line buffers */ 
+	i810ovl->regs->ov0conf = 0; /* two 720 pixel line buffers */
 	
 	i810ovl->regs->ov0cmd = VC_UP_INTERPOLATION | HC_UP_INTERPOLATION | Y_ADJUST |
 		YUV_420;
 	
 	update_overlay(i810drv, i810ovl);
 	
-	/* 
+	/*
 	 * FIXME: If the fence registers are enabled, then the buffer pointers
-	 *        require specific alignment.  This is a problem with planar formats 
+	 *        require specific alignment.  This is a problem with planar formats
 	 *        which have separate pointers for each of the U and V planes.  Packed
-	 *        formats should not be a problem. 
+	 *        formats should not be a problem.
 	 */
 	i810ovl->planar_bug = 0;
 	if (i810_readl(i810drv->mmio_base, FENCE) & 1)
@@ -236,39 +248,11 @@ ovlInitLayer( GraphicsDevice             *device,
 }
 
 static DFBResult
-ovlEnable( DisplayLayer *layer,
-	   void         *driver_data,
-	   void         *layer_data   )
-{
-	I810DriverData       *i810drv = (I810DriverData *) driver_data;
-	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
-
-	i810ovl->regs->ov0cmd |= 1;
-	update_overlay(i810drv, i810ovl);
-	
-	return DFB_OK;
-}
-
-static DFBResult
-ovlDisable( DisplayLayer *layer,
-	    void         *driver_data,
-	    void         *layer_data   )
-{
-	I810DriverData       *i810drv = (I810DriverData *) driver_data;
-	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
-
-	i810ovl->regs->ov0cmd &= ~1;
-	update_overlay(i810drv, i810ovl);
-
-	return DFB_OK;
-}
-
-static DFBResult
-ovlTestConfiguration( DisplayLayer               *layer,
-                      void                       *driver_data,
-                      void                       *layer_data,
-                      DFBDisplayLayerConfig      *config,
-                      DFBDisplayLayerConfigFlags *failed )
+ovlTestRegion( CoreLayer                  *layer,
+               void                       *driver_data,
+               void                       *layer_data,
+               CoreLayerRegionConfig      *config,
+               CoreLayerRegionConfigFlags *failed )
 {
 	DFBDisplayLayerConfigFlags fail = 0;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
@@ -276,7 +260,7 @@ ovlTestConfiguration( DisplayLayer               *layer,
 	if (config->options & ~I810_OVERLAY_SUPPORTED_OPTIONS)
 		fail |= DLCONF_OPTIONS;
 
-	switch (config->pixelformat) {
+	switch (config->format) {
 	case DSPF_I420:
 	case DSPF_YV12:
 	case DSPF_YUY2:
@@ -285,9 +269,9 @@ ovlTestConfiguration( DisplayLayer               *layer,
 	default:
 		fail |= DLCONF_PIXELFORMAT;
 	}
-	if (i810ovl->planar_bug && (config->pixelformat == DSPF_I420 ||
-				    config->pixelformat == DSPF_YV12    )) {
-		DEBUGMSG("Sorry, planar formats will not work when memory tiling "
+	if (i810ovl->planar_bug && (config->format == DSPF_I420 ||
+				    config->format == DSPF_YV12    )) {
+		D_DEBUG("Sorry, planar formats will not work when memory tiling "
 			 "is enabled\n");
 		fail |= DLCONF_PIXELFORMAT;
 	}
@@ -308,115 +292,56 @@ ovlTestConfiguration( DisplayLayer               *layer,
 }
 
 static DFBResult
-ovlSetConfiguration( DisplayLayer          *layer,
-		     void                  *driver_data,
-		     void                  *layer_data,
-		     DFBDisplayLayerConfig *config      )
+ovlSetRegion( CoreLayer                  *layer,
+              void                       *driver_data,
+              void                       *layer_data,
+              void                       *region_data,
+              CoreLayerRegionConfig      *config,
+              CoreLayerRegionConfigFlags  updated,
+              CoreSurface                *surface,
+              CorePalette                *palette )
 {
 	I810DriverData       *i810drv = (I810DriverData *) driver_data;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
 
 	i810ovl->config = *config;
 
-	ovl_calc_regs (i810drv, i810ovl, layer, config);
+	ovl_calc_regs (i810drv, i810ovl, layer, surface, config);
 	update_overlay(i810drv, i810ovl);
 
+	ovlOnOff(i810drv, i810ovl, 1);
+
 	return DFB_OK;
 }
+
 static DFBResult
-ovlSetOpacity( DisplayLayer *layer,
-               void         *driver_data,
-               void         *layer_data,
-               __u8          opacity )
+ovlRemoveRegion( CoreLayer *layer,
+                 void      *driver_data,
+                 void      *layer_data,
+                 void      *region_data )
 {
 	I810DriverData       *i810drv = (I810DriverData *) driver_data;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
 
-	u32 alpha = (u32) (0xFF - opacity);
-
-	switch (dfb_primary_layer_pixelformat()) {
-	case DSPF_ARGB1555:
-	case DSPF_RGB16:
-	case DSPF_RGB24:
-		i810ovl->regs->sclrkm = alpha | (alpha << 8) | (alpha << 16);
-		break;
-	default:
-		return DFB_UNSUPPORTED;
-	}
-
-	i810ovl->regs->sclrkm |= SRC_CONSTANT_ALPHA_BLEND | (3 << 24);
-	
-	update_overlay(i810drv, i810ovl);
+	/* disable overlay */
+	ovlOnOff( i810drv, i810ovl, 0 );
 
 	return DFB_OK;
 }
 
 static DFBResult
-ovlSetScreenLocation( DisplayLayer *layer,
-                      void         *driver_data,
-                      void         *layer_data,
-                      float         x,
-                      float         y,
-                      float         width,
-                      float         height )
-{
-	I810DriverData       *i810drv = (I810DriverData*) driver_data;
-	I810OverlayLayerData *i810ovl = (I810OverlayLayerData*) layer_data;
-     
-	/* get new destination rectangle */
-	dfb_primary_layer_rectangle( x, y, width, height, &i810ovl->dest );
-	
-	ovl_calc_regs( i810drv, i810ovl, layer, &i810ovl->config );
-	update_overlay(i810drv, i810ovl);
-	
-	return DFB_OK;
-}
-
-static DFBResult
-ovlSetDstColorKey( DisplayLayer *layer,
-                   void         *driver_data,
-                   void         *layer_data,
-                   __u8          r,
-                   __u8          g,
-                   __u8          b )
-{
-	I810DriverData       *i810drv = (I810DriverData*) driver_data;
-	I810OverlayLayerData *i810ovl = (I810OverlayLayerData*) layer_data;
-	
-	switch(dfb_primary_layer_pixelformat()) {
-	case DSPF_ARGB1555:
-		i810ovl->regs->dclrkv = PIXEL_ARGB1555(0, r, g, b);
-		break;
-	case DSPF_RGB16:
-		i810ovl->regs->dclrkv = PIXEL_RGB16(r, g, b);
-		break;
-	case DSPF_RGB24:
-		i810ovl->regs->dclrkv = PIXEL_RGB32(r, g, b);
-		break;
-	default:
-		BUG("unexpected pixelformat");
-		return DFB_UNIMPLEMENTED;
-	}
-	i810ovl->regs->dclrkm = 0x80000000;
-
-	update_overlay(i810drv, i810ovl);
-
-	return DFB_OK;
-}
-
-static DFBResult
-ovlFlipBuffers( DisplayLayer        *layer,
+ovlFlipRegion(  CoreLayer           *layer,
                 void                *driver_data,
                 void                *layer_data,
-		DFBSurfaceFlipFlags  flags)
-
+                void                *region_data,
+                CoreSurface         *surface,
+                DFBSurfaceFlipFlags  flags )
 {
 	I810DriverData       *i810drv = (I810DriverData *) driver_data;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData *) layer_data;
-	CoreSurface          *surface = dfb_layer_surface ( layer );
 	u32 current_buffer;
 
-	dfb_surface_flip_buffers( surface );
+	dfb_surface_flip_buffers( surface, false );
 
 	/* select buffer */
 	current_buffer =  (i810ovl->regs->ov0cmd & 4) >> 2;
@@ -428,18 +353,17 @@ ovlFlipBuffers( DisplayLayer        *layer,
 		i810ovl->regs->ov0cmd |= 4;
 	}
 
-	ovl_calc_regs (i810drv, i810ovl, layer, &i810ovl->config); 
+	ovl_calc_regs (i810drv, i810ovl, layer, surface, &i810ovl->config);
 	update_overlay(i810drv, i810ovl);
 	
 	if (flags & DSFLIP_WAIT)
-		dfb_layer_wait_vsync( dfb_layer_at( DLID_PRIMARY ) );
+		dfb_screen_wait_vsync( dfb_screens_at( DSCID_PRIMARY ) );
 
 	return DFB_OK;
 }
 
-
 static DFBResult
-ovlSetColorAdjustment( DisplayLayer       *layer,
+ovlSetColorAdjustment( CoreLayer          *layer,
                        void               *driver_data,
                        void               *layer_data,
                        DFBColorAdjustment *adj )
@@ -447,7 +371,7 @@ ovlSetColorAdjustment( DisplayLayer       *layer,
 	I810DriverData       *i810drv = (I810DriverData*) driver_data;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData*) layer_data;
 	
-	i810ovl->regs->ov0clrc0 = (((adj->brightness >> 8) - 128) & 0xFF) | 
+	i810ovl->regs->ov0clrc0 = (((adj->brightness >> 8) - 128) & 0xFF) |
 		                  ((adj->contrast >> 9) << 8);
 	i810ovl->regs->ov0clrc1 = (adj->saturation >> 8) & 0xFF;
 
@@ -456,10 +380,11 @@ ovlSetColorAdjustment( DisplayLayer       *layer,
 }
 	
 static DFBResult
-ovlSetField( DisplayLayer *layer,
-	     void         *driver_data,
-	     void         *layer_data,
-	     int           field )
+ovlSetInputField( CoreLayer *layer,
+		  void      *driver_data,
+		  void      *layer_data,
+		  void      *region_data,
+		  int        field )
 {
 	I810DriverData       *i810drv = (I810DriverData*) driver_data;
 	I810OverlayLayerData *i810ovl = (I810OverlayLayerData*) layer_data;
@@ -472,43 +397,40 @@ ovlSetField( DisplayLayer *layer,
 }
 
 DisplayLayerFuncs i810OverlayFuncs = {
-     .LayerDataSize      ovlLayerDataSize,
-     .InitLayer          ovlInitLayer,
-     .Enable             ovlEnable,
-     .Disable            ovlDisable,
-     .TestConfiguration  ovlTestConfiguration,
-     .SetConfiguration   ovlSetConfiguration,
-     .SetOpacity         ovlSetOpacity,
-     .SetScreenLocation  ovlSetScreenLocation,
-     .SetDstColorKey     ovlSetDstColorKey,
-     .FlipBuffers        ovlFlipBuffers,
-     .SetColorAdjustment ovlSetColorAdjustment,
-     .SetField           ovlSetField
+     LayerDataSize:      ovlLayerDataSize,
+     InitLayer:          ovlInitLayer,
+     TestRegion:         ovlTestRegion,
+     SetRegion:          ovlSetRegion,
+     RemoveRegion:       ovlRemoveRegion,
+     FlipRegion:         ovlFlipRegion,
+     SetColorAdjustment: ovlSetColorAdjustment,
+     SetInputField:      ovlSetInputField
+
 };
 
 
-static void ovl_calc_regs (I810DriverData        *i810drv, 
+static void ovl_calc_regs (I810DriverData        *i810drv,
 			   I810OverlayLayerData  *i810ovl,
-			   DisplayLayer          *layer,
-			   DFBDisplayLayerConfig *config  )
+			   CoreLayer             *layer,
+			   CoreSurface           *surface,
+			   CoreLayerRegionConfig *config )
 {
 	u32 swidth = 0, y_offset, v_offset = 0, u_offset = 0;
 	u32 drw_w, src_w, drw_h, src_h, xscaleInt, xscaleFract, yscaleInt;
 	u32 xscaleFractUV = 0, xscaleIntUV, yscaleIntUV = 0, yscaleFract, yscaleFractUV = 0;
 
-	CoreSurface   *surface      = dfb_layer_surface( layer );
 	SurfaceBuffer *front_buffer = surface->front_buffer;
 
-	drw_w = i810ovl->dest.w;
-	drw_h = i810ovl->dest.h;
+	drw_w = config->dest.w;
+	drw_h = config->dest.h;
 	src_w = surface->width;
 	src_h = surface->height;
 
-	if (config->options & DLOP_DEINTERLACING) 
-		src_h >>= 1; 
+	if (config->options & DLOP_DEINTERLACING)
+		src_h >>= 1;
 
         /* reset command register except the enable bit and buffer select bits */
-	i810ovl->regs->ov0cmd &= 7; 
+	i810ovl->regs->ov0cmd &= 7;
 
 	/* Set source dimension in bytes */
 	switch (surface->format) {
@@ -536,11 +458,11 @@ static void ovl_calc_regs (I810DriverData        *i810drv,
 		i810ovl->regs->ov0conf = 0;
 
 	/* set dest window position and dimension */
-	i810ovl->regs->dwinpos = (i810ovl->dest.y << 16) | i810ovl->dest.x;
+	i810ovl->regs->dwinpos = (config->dest.y << 16) | config->dest.x;
 	i810ovl->regs->dwinsz = (drw_h << 16) | drw_w;
 
 	/* Set buffer pointers */
-	y_offset = (dfb_gfxcard_memory_physical(NULL, front_buffer->video.offset)); 
+	y_offset = (dfb_gfxcard_memory_physical(NULL, front_buffer->video.offset));
 
 	switch (surface->format) {
 	case DSPF_I420:
@@ -570,46 +492,46 @@ static void ovl_calc_regs (I810DriverData        *i810drv,
 	i810ovl->regs->yrgbscale = 0x80004000;
 	i810ovl->regs->uvscale = 0x80004000;
 
-	i810ovl->regs->ov0cmd |= VC_UP_INTERPOLATION | HC_UP_INTERPOLATION | Y_ADJUST | FRAME_MODE; 
+	i810ovl->regs->ov0cmd |= VC_UP_INTERPOLATION | HC_UP_INTERPOLATION | Y_ADJUST | FRAME_MODE;
 
-	if (config->options & DLOP_DEINTERLACING) 
+	if (config->options & DLOP_DEINTERLACING)
 		i810ovl->regs->ov0cmd |= FIELD_MODE;
 
-	if ((drw_w != src_w) || (drw_h != src_h)) 
+	if ((drw_w != src_w) || (drw_h != src_h))
 	{
 		xscaleInt = (src_w / drw_w) & 0x3;
 		xscaleFract = (src_w << 12) / drw_w;
 		yscaleInt = (src_h / drw_h) & 0x3;
 		yscaleFract = (src_h << 12) / drw_h;
 
-		i810ovl->regs->yrgbscale = (xscaleInt << 15) | 
+		i810ovl->regs->yrgbscale = (xscaleInt << 15) |
 			((xscaleFract & 0xFFF) << 3) |
 			(yscaleInt) |
 			((yscaleFract & 0xFFF) << 20);
 
-		if (drw_w > src_w) 
+		if (drw_w > src_w)
 		{
 			i810ovl->regs->ov0cmd &= ~HORIZONTAL_CHROMINANCE_FILTER;
 			i810ovl->regs->ov0cmd &= ~HORIZONTAL_LUMINANCE_FILTER;
 			i810ovl->regs->ov0cmd |= (HC_UP_INTERPOLATION | HL_UP_INTERPOLATION);
 		}
 
-		if (drw_h > src_h) 
-		{ 
+		if (drw_h > src_h)
+		{
 			i810ovl->regs->ov0cmd &= ~VERTICAL_CHROMINANCE_FILTER;
 			i810ovl->regs->ov0cmd &= ~VERTICAL_LUMINANCE_FILTER;
 			i810ovl->regs->ov0cmd |= (VC_UP_INTERPOLATION | VL_UP_INTERPOLATION);
 		}
 
-		if (drw_w < src_w) 
-		{ 
+		if (drw_w < src_w)
+		{
 			i810ovl->regs->ov0cmd &= ~HORIZONTAL_CHROMINANCE_FILTER;
 			i810ovl->regs->ov0cmd &= ~HORIZONTAL_LUMINANCE_FILTER;
 			i810ovl->regs->ov0cmd |= (HC_DOWN_INTERPOLATION | HL_DOWN_INTERPOLATION);
 		}
 		
-		if (drw_h < src_h) 
-		{ 
+		if (drw_h < src_h)
+		{
 			i810ovl->regs->ov0cmd &= ~VERTICAL_CHROMINANCE_FILTER;
 			i810ovl->regs->ov0cmd &= ~VERTICAL_LUMINANCE_FILTER;
 			i810ovl->regs->ov0cmd |= (VC_DOWN_INTERPOLATION | VL_DOWN_INTERPOLATION);
@@ -674,7 +596,7 @@ static void ovl_calc_regs (I810DriverData        *i810drv,
 			i810ovl->regs->ov0cmd |= Y_SWAP;
 		break;
 	default:
-		BUG("unexpected pixelformat");
+		D_BUG("unexpected pixelformat");
 		break;
 	}
 
