@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,17 +37,21 @@
 
 #include <directfb.h>
 
-#include <core/fusion/shmalloc.h>
+#include <direct/messages.h>
+
+#include <fusion/shmalloc.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/gfxcard.h>
+#include <core/layer_control.h>
 #include <core/layers.h>
+#include <core/screen.h>
 #include <core/surfaces.h>
 #include <core/system.h>
 #include <core/windows.h>
 
-#include <misc/mem.h>
+#include <direct/mem.h>
 
 
 #include "regs.h"
@@ -54,15 +59,7 @@
 #include "matrox.h"
 
 typedef struct {
-     bool                  listener_running;
-     DFBDisplayLayerConfig config;
-
-     struct {
-          float            x;
-          float            y;
-          float            w;
-          float            h;
-     } location;
+     CoreLayerRegionConfig config;
 
      /* Stored registers */
      struct {
@@ -103,7 +100,7 @@ typedef struct {
 static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                           bool onsync );
 static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
-                           DisplayLayer *layer, DFBDisplayLayerConfig *config );
+                           CoreLayerRegionConfig *config, CoreSurface *surface );
 
 #define BES_SUPPORTED_OPTIONS   (DLOP_DEINTERLACING | DLOP_DST_COLORKEY)
 
@@ -115,56 +112,47 @@ besLayerDataSize()
 {
      return sizeof(MatroxBesLayerData);
 }
-     
+
 static DFBResult
-besInitLayer( GraphicsDevice             *device,
-              DisplayLayer               *layer,
-              DisplayLayerInfo           *layer_info,
-              DFBDisplayLayerConfig      *default_config,
-              DFBColorAdjustment         *default_adj,
+besInitLayer( CoreLayer                  *layer,
               void                       *driver_data,
-              void                       *layer_data )
+              void                       *layer_data,
+              DFBDisplayLayerDescription *description,
+              DFBDisplayLayerConfig      *config,
+              DFBColorAdjustment         *adjustment )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
-     MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
      volatile __u8      *mmio = mdrv->mmio_base;
-     
+
      /* set capabilities and type */
-     layer_info->desc.caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
-                             DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST |
-                             DLCAPS_DEINTERLACING | DLCAPS_DST_COLORKEY;
-     layer_info->desc.type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
+     description->caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
+                         DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST |
+                         DLCAPS_DEINTERLACING | DLCAPS_DST_COLORKEY;
+     description->type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
 
      /* set name */
-     snprintf( layer_info->desc.name,
+     snprintf( description->name,
                DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "Matrox Backend Scaler" );
 
      /* fill out the default configuration */
-     default_config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
-                                   DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
-                                   DLCONF_OPTIONS;
-     default_config->width       = 640;
-     default_config->height      = 480;
-     default_config->pixelformat = DSPF_YUY2;
-     default_config->buffermode  = DLBM_FRONTONLY;
-     default_config->options     = DLOP_NONE;
+     config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
+                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
+                           DLCONF_OPTIONS;
+     config->width       = 640;
+     config->height      = 480;
+     config->pixelformat = DSPF_YUY2;
+     config->buffermode  = DLBM_FRONTONLY;
+     config->options     = DLOP_NONE;
 
      /* fill out default color adjustment,
         only fields set in flags will be accepted from applications */
-     default_adj->flags      = DCAF_BRIGHTNESS | DCAF_CONTRAST;
-     default_adj->brightness = 0x8000;
-     default_adj->contrast   = 0x8000;
-     
-     
-     /* initialize destination rectangle */
-     mbes->location.x = 0.125f;
-     mbes->location.y = 0.125f;
-     mbes->location.w = 0.25f;
-     mbes->location.h = 0.25f;
-     
+     adjustment->flags      = DCAF_BRIGHTNESS | DCAF_CONTRAST;
+     adjustment->brightness = 0x8000;
+     adjustment->contrast   = 0x8000;
+
      /* disable backend scaler */
      mga_out32( mmio, 0, BESCTL );
-     
+
      /* set defaults */
      mga_out_dac( mmio, XKEYOPMODE, 0x00 ); /* keying off */
 
@@ -177,73 +165,34 @@ besInitLayer( GraphicsDevice             *device,
      mga_out_dac( mmio, XCOLKEY0BLUE,  0x00 );
 
      mga_out32( mmio, 0x80, BESLUMACTL );
-     
-     return DFB_OK;
-}
-
-
-static void
-besOnOff( MatroxDriverData   *mdrv,
-          MatroxBesLayerData *mbes,
-          int                 on )
-{
-     if (on)
-          mbes->regs.besCTL |= 1;
-     else
-          mbes->regs.besCTL &= ~1;
-
-     mga_out32( mdrv->mmio_base,
-                mbes->regs.besCTL | mbes->regs.besCTL_field, BESCTL );
-}
-
-static DFBResult
-besEnable( DisplayLayer *layer,
-           void         *driver_data,
-           void         *layer_data )
-{
-     MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
-     MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-     
-     /* enable backend scaler */
-     besOnOff( mdrv, mbes, 1 );
 
      return DFB_OK;
 }
 
 static DFBResult
-besDisable( DisplayLayer *layer,
-            void         *driver_data,
-            void         *layer_data )
+besTestRegion( CoreLayer                  *layer,
+               void                       *driver_data,
+               void                       *layer_data,
+               CoreLayerRegionConfig      *config,
+               CoreLayerRegionConfigFlags *failed )
 {
-     MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
-     MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-
-     /* disable backend scaler */
-     besOnOff( mdrv, mbes, 0 );
-
-     return DFB_OK;
-}
-
-static DFBResult
-besTestConfiguration( DisplayLayer               *layer,
-                      void                       *driver_data,
-                      void                       *layer_data,
-                      DFBDisplayLayerConfig      *config,
-                      DFBDisplayLayerConfigFlags *failed )
-{
-     int                         max_width = 1024;
-     DFBDisplayLayerConfigFlags  fail      = 0;
      MatroxDriverData           *mdrv      = (MatroxDriverData*) driver_data;
+     MatroxDeviceData           *mdev      = mdrv->device_data;
+     int                         max_width = mdev->g450_matrox ? 2048 : 1024;
+     CoreLayerRegionConfigFlags  fail      = 0;
 
      if (config->options & ~BES_SUPPORTED_OPTIONS)
-          fail |= DLCONF_OPTIONS;
+          fail |= CLRCF_OPTIONS;
 
-     switch (config->pixelformat) {
+     switch (config->format) {
           case DSPF_YUY2:
+          case DSPF_NV12:
+          case DSPF_NV21:
                break;
 
           case DSPF_RGB32:
-               max_width = 512;
+               if (!mdev->g450_matrox)
+                    max_width = 512;
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_UYVY:
@@ -253,14 +202,29 @@ besTestConfiguration( DisplayLayer               *layer,
                if (mdrv->accelerator != FB_ACCEL_MATROX_MGAG200)
                     break;
           default:
-               fail |= DLCONF_PIXELFORMAT;
+               fail |= CLRCF_FORMAT;
+     }
+
+     switch (config->format) {
+          case DSPF_I420:
+          case DSPF_YV12:
+          case DSPF_NV12:
+          case DSPF_NV21:
+               if (config->height & 1)
+                    fail |= CLRCF_HEIGHT;
+          case DSPF_YUY2:
+          case DSPF_UYVY:
+               if (config->width & 1)
+                    fail |= CLRCF_WIDTH;
+          default:
+               break;
      }
 
      if (config->width > max_width || config->width < 1)
-          fail |= DLCONF_WIDTH;
+          fail |= CLRCF_WIDTH;
 
      if (config->height > 1024 || config->height < 1)
-          fail |= DLCONF_HEIGHT;
+          fail |= CLRCF_HEIGHT;
 
      if (failed)
           *failed = fail;
@@ -272,173 +236,140 @@ besTestConfiguration( DisplayLayer               *layer,
 }
 
 static DFBResult
-besSetConfiguration( DisplayLayer          *layer,
-                     void                  *driver_data,
-                     void                  *layer_data,
-                     DFBDisplayLayerConfig *config )
+besSetRegion( CoreLayer                  *layer,
+              void                       *driver_data,
+              void                       *layer_data,
+              void                       *region_data,
+              CoreLayerRegionConfig      *config,
+              CoreLayerRegionConfigFlags  updated,
+              CoreSurface                *surface,
+              CorePalette                *palette )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
+     volatile __u8      *mmio = mdrv->mmio_base;
 
      /* remember configuration */
      mbes->config = *config;
-     
-     bes_calc_regs( mdrv, mbes, layer, config );
-     bes_set_regs( mdrv, mbes, true );
 
-     return DFB_OK;
-}
+     /* set main configuration */
+     if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT |
+                    CLRCF_OPTIONS | CLRCF_DEST | CLRCF_OPACITY | CLRCF_SOURCE))
+     {
+          bes_calc_regs( mdrv, mbes, config, surface );
+          bes_set_regs( mdrv, mbes, true );
+     }
 
-static DFBResult
-besSetOpacity( DisplayLayer *layer,
-               void         *driver_data,
-               void         *layer_data,
-               __u8          opacity )
-{
-     MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
-     MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-     
-     switch (opacity) {
-          case 0:
-               besOnOff( mdrv, mbes, 0 );
-               break;
-          case 0xFF:
-               besOnOff( mdrv, mbes, 1 );
-               break;
-          default:
-               return DFB_UNSUPPORTED;
+     /* set color key */
+     if (updated & CLRCF_DSTKEY) {
+          DFBColor key = config->dst_key;
+
+          switch (dfb_primary_layer_pixelformat()) {
+               case DSPF_ARGB1555:
+                    key.r >>= 3;
+                    key.g >>= 3;
+                    key.b >>= 3;
+                    break;
+
+               case DSPF_RGB16:
+                    key.r >>= 3;
+                    key.g >>= 2;
+                    key.b >>= 3;
+                    break;
+
+               default:
+                    ;
+          }
+
+          mga_out_dac( mmio, XCOLKEY0RED,   key.r );
+          mga_out_dac( mmio, XCOLKEY0GREEN, key.g );
+          mga_out_dac( mmio, XCOLKEY0BLUE,  key.b );
      }
 
      return DFB_OK;
 }
 
 static DFBResult
-besSetScreenLocation( DisplayLayer *layer,
-                      void         *driver_data,
-                      void         *layer_data,
-                      float         x,
-                      float         y,
-                      float         width,
-                      float         height )
+besRemoveRegion( CoreLayer *layer,
+                 void      *driver_data,
+                 void      *layer_data,
+                 void      *region_data )
+{
+     MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
+
+     /* disable backend scaler */
+     mga_out32( mdrv->mmio_base, 0, BESCTL );
+
+     return DFB_OK;
+}
+
+static DFBResult
+besFlipRegion( CoreLayer           *layer,
+               void                *driver_data,
+               void                *layer_data,
+               void                *region_data,
+               CoreSurface         *surface,
+               DFBSurfaceFlipFlags  flags )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-     
-     /* set new destination rectangle */
-     mbes->location.x = x;
-     mbes->location.y = y;
-     mbes->location.w = width;
-     mbes->location.h = height;
 
-     bes_calc_regs( mdrv, mbes, layer, &mbes->config );
-     bes_set_regs( mdrv, mbes, true );
-     
-     return DFB_OK;
-}
+     dfb_surface_flip_buffers( surface, false );
 
-static DFBResult
-besSetDstColorKey( DisplayLayer *layer,
-                   void         *driver_data,
-                   void         *layer_data,
-                   __u8          r,
-                   __u8          g,
-                   __u8          b )
-{
-     MatroxDriverData *mdrv = (MatroxDriverData*) driver_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
-     
-     switch (dfb_primary_layer_pixelformat()) {
-          case DSPF_ARGB1555:
-               r >>= 3;
-               g >>= 3;
-               b >>= 3;
-               break;
-          
-          case DSPF_RGB16:
-               r >>= 3;
-               g >>= 2;
-               b >>= 3;
-               break;
-          
-          default:
-               ;
-     }
-     
-     mga_out_dac( mmio, XCOLKEY0RED,   r );
-     mga_out_dac( mmio, XCOLKEY0GREEN, g );
-     mga_out_dac( mmio, XCOLKEY0BLUE,  b );
-
-     return DFB_OK;
-}
-
-static DFBResult
-besFlipBuffers( DisplayLayer        *layer,
-                void                *driver_data,
-                void                *layer_data,
-                DFBSurfaceFlipFlags  flags )
-{
-     MatroxDriverData   *mdrv    = (MatroxDriverData*) driver_data;
-     MatroxBesLayerData *mbes    = (MatroxBesLayerData*) layer_data;
-     CoreSurface        *surface = dfb_layer_surface( layer );
-     
-     dfb_surface_flip_buffers( surface );
-     
-     bes_calc_regs( mdrv, mbes, layer, &mbes->config );
+     bes_calc_regs( mdrv, mbes, &mbes->config, surface );
      bes_set_regs( mdrv, mbes, flags & DSFLIP_ONSYNC );
 
      if (flags & DSFLIP_WAIT)
-          dfb_layer_wait_vsync( dfb_layer_at( DLID_PRIMARY ) );
+          dfb_screen_wait_vsync( mdrv->primary );
 
      return DFB_OK;
 }
 
 static DFBResult
-besSetColorAdjustment( DisplayLayer       *layer,
+besSetColorAdjustment( CoreLayer          *layer,
                        void               *driver_data,
                        void               *layer_data,
                        DFBColorAdjustment *adj )
 {
      MatroxDriverData *mdrv = (MatroxDriverData*) driver_data;
      volatile __u8    *mmio = mdrv->mmio_base;
-     
+
      mga_out32( mmio, (adj->contrast >> 8) |
                       ((__u8)(((int)adj->brightness >> 8) - 128)) << 16,
                 BESLUMACTL );
-     
+
      return DFB_OK;
 }
 
 static DFBResult
-besSetField( DisplayLayer *layer,
-             void         *driver_data,
-             void         *layer_data,
-             int           field )
+besSetInputField( CoreLayer *layer,
+                  void      *driver_data,
+                  void      *layer_data,
+                  void      *region_data,
+                  int        field )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-     
+
      mbes->regs.besCTL_field = field ? 0x2000000 : 0;
 
      mga_out32( mdrv->mmio_base,
                 mbes->regs.besCTL | mbes->regs.besCTL_field, BESCTL );
-     
+
      return DFB_OK;
 }
 
-
 DisplayLayerFuncs matroxBesFuncs = {
      LayerDataSize:      besLayerDataSize,
      InitLayer:          besInitLayer,
-     Enable:             besEnable,
-     Disable:            besDisable,
-     TestConfiguration:  besTestConfiguration,
-     SetConfiguration:   besSetConfiguration,
-     SetOpacity:         besSetOpacity,
-     SetScreenLocation:  besSetScreenLocation,
-     SetDstColorKey:     besSetDstColorKey,
-     FlipBuffers:        besFlipBuffers,
+
+     TestRegion:         besTestRegion,
+     SetRegion:          besSetRegion,
+     RemoveRegion:       besRemoveRegion,
+     FlipRegion:         besFlipRegion,
+
      SetColorAdjustment: besSetColorAdjustment,
-     SetField:           besSetField
+     SetInputField:      besSetInputField
 };
 
 
@@ -453,12 +384,16 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
      if (!onsync) {
           VideoMode *current_mode = dfb_system_current_mode();
 
+          /* FIXME: I don't think this should be NULL ever. */
+          if (!current_mode)
+               return;
+
           line = mga_in32( mmio, MGAREG_VCOUNT ) + 48;
 
           if (line > current_mode->yres)
                line = current_mode->yres;
      }
-     
+
      mga_out32( mmio, mbes->regs.besGLOBCTL | (line << 16), BESGLOBCTL);
 
      mga_out32( mmio, mbes->regs.besA1ORG, BESA1ORG );
@@ -494,49 +429,103 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
      mga_out_dac( mmio, XKEYOPMODE, mbes->regs.xKEYOPMODE );
 }
 
-static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
-                           DisplayLayer *layer, DFBDisplayLayerConfig *config )
+static void bes_calc_regs( MatroxDriverData      *mdrv,
+                           MatroxBesLayerData    *mbes,
+                           CoreLayerRegionConfig *config,
+                           CoreSurface           *surface )
 {
-     int tmp, hzoom, intrep;
-
-     DFBRectangle   dest;
+     MatroxDeviceData *mdev = mdrv->device_data;
+     int cropleft, cropright, croptop, cropbot, croptop_uv;
+     int pitch, tmp, hzoom, intrep, field_height;
+     DFBRectangle   source, dest;
      DFBRegion      dstBox;
-     int            drw_w, drw_h;
-     int            field_height;
-     CoreSurface   *surface      = dfb_layer_surface( layer );
      SurfaceBuffer *front_buffer = surface->front_buffer;
      VideoMode     *current_mode = dfb_system_current_mode();
 
+     /* FIXME: I don't think this should be NULL ever. */
+     if (!current_mode)
+          return;
+
+     source = config->source;
+     dest   = config->dest;
+
+     if (!mdev->g450_matrox && surface->format == DSPF_RGB32)
+          dest.w = source.w;
+
+     pitch = front_buffer->video.pitch;
+
+     field_height = surface->height;
+
+     if (config->options & DLOP_DEINTERLACING) {
+          field_height /= 2;
+          source.y /= 2;
+          source.h /= 2;
+          pitch    *= 2;
+     } else
+          mbes->regs.besCTL_field = 0;
+
+     /* calculate destination cropping */
+     cropleft  = -dest.x;
+     croptop   = -dest.y;
+     cropright = dest.x + dest.w - current_mode->xres;
+     cropbot   = dest.y + dest.h - current_mode->yres;
+
+     cropleft   = cropleft > 0 ? cropleft : 0;
+     croptop    = croptop > 0 ? croptop : 0;
+     cropright  = cropright > 0 ? cropright : 0;
+     cropbot    = cropbot > 0 ? cropbot : 0;
+     croptop_uv = croptop;
+
      /* destination box */
-     dfb_primary_layer_rectangle( mbes->location.x, mbes->location.y,
-                                  mbes->location.w, mbes->location.h, &dest );
-     
-     dstBox.x1 = dest.x;
-     dstBox.y1 = dest.y;
-     dstBox.x2 = dest.x + dest.w;
-     dstBox.y2 = dest.y + dest.h;
+     dstBox.x1 = dest.x + cropleft;
+     dstBox.y1 = dest.y + croptop;
+     dstBox.x2 = dest.x + dest.w - 1 - cropright;
+     dstBox.y2 = dest.y + dest.h - 1 - cropbot;
+
+     /* scale crop values to source dimensions */
+     if (cropleft)
+          cropleft = ((__u64) (source.w << 16) * cropleft / dest.w) & ~0x3;
+     if (croptop)
+          croptop = ((__u64) (source.h << 16) * croptop / dest.h) & ~0x3;
+     if (cropright)
+          cropright = ((__u64) (source.w << 16) * cropright / dest.w) & ~0x3;
+     if (cropbot)
+          cropbot = ((__u64) (source.h << 16) * cropbot / dest.h) & ~0x3;
+     if (croptop_uv)
+          croptop_uv = ((__u64) ((source.h/2) << 16) * croptop_uv / dest.h) & ~0x3;
 
-     /* destination size */
-     drw_w = dest.w;
-     drw_h = dest.h;
-     
      /* should horizontal zoom be used? */
-     hzoom = (1000000/current_mode->pixclock >= 135) ? 1 : 0;
+     if (mdev->g450_matrox)
+          hzoom = (1000000/current_mode->pixclock >= 234) ? 1 : 0;
+     else
+          hzoom = (1000000/current_mode->pixclock >= 135) ? 1 : 0;
 
      /* initialize */
      mbes->regs.besGLOBCTL = 0;
 
-     /* clear everything but the enable bit that may be set */
-     mbes->regs.besCTL &= 1;
+     /* enable/disable depending on opacity */
+     mbes->regs.besCTL = config->opacity ? 1 : 0;
 
      /* pixel format settings */
      switch (surface->format) {
-          case DSPF_I420:
           case DSPF_YV12:
+               mbes->regs.besGLOBCTL |= BESCORDER;
+               /* fall through */
+
+          case DSPF_I420:
                mbes->regs.besGLOBCTL |= BESPROCAMP | BES3PLANE;
                mbes->regs.besCTL     |= BESHFEN | BESVFEN | BESCUPS | BES420PL;
                break;
 
+          case DSPF_NV21:
+               mbes->regs.besGLOBCTL |= BESCORDER;
+               /* fall through */
+
+          case DSPF_NV12:
+               mbes->regs.besGLOBCTL |= BESPROCAMP;
+               mbes->regs.besCTL     |= BESHFEN | BESVFEN | BESCUPS | BES420PL;
+               break;
+
           case DSPF_UYVY:
                mbes->regs.besGLOBCTL |= BESUYVYFMT;
                /* fall through */
@@ -556,43 +545,44 @@ static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
 
           case DSPF_RGB32:
                mbes->regs.besGLOBCTL |= BESRGB32;
-
-               drw_w = surface->width;
-               dstBox.x2 = dstBox.x1 + surface->width;
                break;
 
           default:
-               BUG( "unexpected pixelformat" );
+               D_BUG( "unexpected pixelformat" );
                return;
      }
 
      mbes->regs.besGLOBCTL |= 3*hzoom | (current_mode->yres & 0xFFF) << 16;
-     mbes->regs.besA1ORG    = front_buffer->video.offset;
-     mbes->regs.besA2ORG    = front_buffer->video.offset +
-                              front_buffer->video.pitch;
+
+     mbes->regs.besPITCH = pitch / DFB_BYTES_PER_PIXEL(surface->format);
+
+     /* buffer offsets */
+     mbes->regs.besA1ORG = front_buffer->video.offset +
+                           pitch * (source.y + (croptop >> 16));
+     mbes->regs.besA2ORG = mbes->regs.besA1ORG +
+                           front_buffer->video.pitch;
 
      switch (surface->format) {
-          case DSPF_I420:
-               mbes->regs.besA1CORG  = mbes->regs.besA1ORG + surface->height *
-                                       front_buffer->video.pitch;
+          case DSPF_NV12:
+          case DSPF_NV21:
+               mbes->regs.besA1CORG  = front_buffer->video.offset +
+                                       surface->height * front_buffer->video.pitch +
+                                       pitch * (source.y/2 + (croptop_uv >> 16));
                mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
-                                       front_buffer->video.pitch/2;
-
-               mbes->regs.besA1C3ORG = mbes->regs.besA1CORG + surface->height/2 *
-                                       front_buffer->video.pitch/2;
-               mbes->regs.besA2C3ORG = mbes->regs.besA1C3ORG +
-                                       front_buffer->video.pitch/2;
+                                       front_buffer->video.pitch;
                break;
 
+          case DSPF_I420:
           case DSPF_YV12:
-               mbes->regs.besA1C3ORG = mbes->regs.besA1ORG + surface->height *
-                                       front_buffer->video.pitch;
-               mbes->regs.besA2C3ORG = mbes->regs.besA1C3ORG +
+               mbes->regs.besA1CORG  = front_buffer->video.offset +
+                                       surface->height * front_buffer->video.pitch +
+                                       pitch/2 * (source.y/2 + (croptop_uv >> 16));
+               mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
                                        front_buffer->video.pitch/2;
 
-               mbes->regs.besA1CORG  = mbes->regs.besA1C3ORG + surface->height/2 *
-                                       front_buffer->video.pitch/2;
-               mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
+               mbes->regs.besA1C3ORG = mbes->regs.besA1CORG +
+                                       surface->height/2 * front_buffer->video.pitch/2;
+               mbes->regs.besA2C3ORG = mbes->regs.besA1C3ORG +
                                        front_buffer->video.pitch/2;
                break;
 
@@ -600,43 +590,56 @@ static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                ;
      }
 
-     mbes->regs.besHCOORD   = (dstBox.x1 << 16) | (dstBox.x2 - 1);
-     mbes->regs.besVCOORD   = (dstBox.y1 << 16) | (dstBox.y2 - 1);
+     mbes->regs.besHCOORD   = (dstBox.x1 << 16) | dstBox.x2;
+     mbes->regs.besVCOORD   = (dstBox.y1 << 16) | dstBox.y2;
 
-     mbes->regs.besHSRCST   = 0;
-     mbes->regs.besHSRCEND  = (surface->width - 1) << 16;
+     mbes->regs.besHSRCST   = (source.x << 16) + cropleft;
+     mbes->regs.besHSRCEND  = ((source.x + source.w - 1) << 16) - cropright;
      mbes->regs.besHSRCLST  = (surface->width - 1) << 16;
 
-     mbes->regs.besV1WGHT   = 0;
-     mbes->regs.besV2WGHT   = 0x18000;
-
-     mbes->regs.besV1SRCLST = surface->height - 1;
-     mbes->regs.besV2SRCLST = surface->height - 2;
-
-     mbes->regs.besPITCH    = front_buffer->video.pitch /
-                              DFB_BYTES_PER_PIXEL(surface->format);
-
-     field_height           = surface->height;
-
-     if (config->options & DLOP_DEINTERLACING) {
-          field_height        /= 2;
-          mbes->regs.besPITCH *= 2;
+     /* vertical starting weights */
+     tmp = croptop & 0xfffc;
+     mbes->regs.besV1WGHT = tmp;
+     if (tmp >= 0x8000) {
+          tmp = tmp - 0x8000;
+          /* fields start on the same line */
+          if ((source.y + (croptop >> 16)) & 1)
+               mbes->regs.besCTL |= BESV1SRCSTP | BESV2SRCSTP;
+     } else {
+          tmp = 0x10000 | (0x8000 - tmp);
+          /* fields start on alternate lines */
+          if ((source.y + (croptop >> 16)) & 1)
+               mbes->regs.besCTL |= BESV1SRCSTP;
+          else
+               mbes->regs.besCTL |= BESV2SRCSTP;
      }
-     else
-          mbes->regs.besCTL_field = 0;
+     mbes->regs.besV2WGHT = tmp;
 
-     if (config->pixelformat == DSPF_RGB32)
-          mbes->regs.besHISCAL = 0x20000;
-     else {
-          intrep = ((drw_w == surface->width) || (drw_w < 2)) ? 0 : 1;
-          tmp = (((surface->width - intrep) << 16) / (drw_w - intrep)) << hzoom;
+     mbes->regs.besV1SRCLST = mbes->regs.besV2SRCLST =
+          field_height - 1 - source.y - (croptop >> 16);
+
+     /* horizontal scaling */
+     if (!mdev->g450_matrox && surface->format == DSPF_RGB32) {
+          mbes->regs.besHISCAL   = 0x20000 << hzoom;
+          mbes->regs.besHSRCST  *= 2;
+          mbes->regs.besHSRCEND *= 2;
+          mbes->regs.besHSRCLST *= 2;
+          mbes->regs.besPITCH   *= 2;
+     } else {
+          intrep = ((mbes->regs.besCTL & BESHFEN) || (source.w > dest.w)) ? 1 : 0;
+          if ((dest.w == source.w) || (dest.w < 2))
+               intrep = 0;
+          tmp = (((source.w - intrep) << 16) / (dest.w - intrep)) << hzoom;
           if (tmp >= (32 << 16))
                tmp = (32 << 16) - 1;
           mbes->regs.besHISCAL = tmp & 0x001ffffc;
      }
-     
-     intrep = ((drw_h == field_height) || (drw_h < 2)) ? 0 : 1;
-     tmp = ((field_height - intrep) << 16) / (drw_h - intrep);
+
+     /* vertical scaling */
+     intrep = ((mbes->regs.besCTL & BESVFEN) || (source.h > dest.h)) ? 1 : 0;
+     if ((dest.h == source.h) || (dest.h < 2))
+          intrep = 0;
+     tmp = ((source.h - intrep) << 16) / (dest.h - intrep);
      if(tmp >= (32 << 16))
           tmp = (32 << 16) - 1;
      mbes->regs.besVISCAL = tmp & 0x001ffffc;
@@ -644,4 +647,3 @@ static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
      /* enable color keying? */
      mbes->regs.xKEYOPMODE = (config->options & DLOP_DST_COLORKEY) ? 1 : 0;
 }
-
