@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -26,12 +27,16 @@
 
 #include <directfb.h>
 
+#include <direct/messages.h>
+#include <direct/util.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
 #include <core/state.h>
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
+#include <core/palette.h>
 
 #include <gfx/convert.h>
 #include <misc/util.h>
@@ -50,48 +55,55 @@ void matrox_set_destination( MatroxDriverData *mdrv,
 {
      volatile __u8 *mmio            = mdrv->mmio_base;
      SurfaceBuffer *buffer          = destination->back_buffer;
-     int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(destination->format);
+     SurfaceBuffer *depth_buffer    = destination->depth_buffer;
+     int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(buffer->format);
 
-     mdev->planar    = DFB_PLANAR_PIXELFORMAT( destination->format );
      mdev->dst_pitch = buffer->video.pitch / bytes_per_pixel;
+     mdev->dst_format = destination->format;
+
+     D_ASSERT( mdev->dst_pitch % 32 == 0 );
+
+     D_ASSERT( buffer->video.offset % 64 == 0 );
+
+     mdev->depth_buffer = depth_buffer != NULL;
+
+     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
+          mdev->dst_pitch /= 2;
 
      if (mdev->old_matrox) {
-          mdev->dst_offset = buffer->video.offset / bytes_per_pixel;
-          if (destination->format == DSPF_I420) {
-               mdev->dst_cb_offset = mdev->dst_offset + destination->height * mdev->dst_pitch;
-               mdev->dst_cr_offset = mdev->dst_cb_offset + destination->height * mdev->dst_pitch / 4;
-          } else if (destination->format == DSPF_YV12) {
-               mdev->dst_cr_offset = mdev->dst_offset + destination->height * mdev->dst_pitch;
-               mdev->dst_cb_offset = mdev->dst_cr_offset + destination->height * mdev->dst_pitch / 4;
-          }
+          mdev->dst_offset[0] = buffer->video.offset / bytes_per_pixel;
      } else {
-          mdev->dst_offset = dfb_gfxcard_memory_physical( NULL, buffer->video.offset ) & 0x1FFFFFF;
+          mdev->dst_offset[0] = mdev->fb.offset + buffer->video.offset;
           if (destination->format == DSPF_I420) {
-               mdev->dst_cb_offset = mdev->dst_offset + destination->height * buffer->video.pitch;
-               mdev->dst_cr_offset = mdev->dst_cb_offset + destination->height * buffer->video.pitch / 4;
+               mdev->dst_offset[1] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
+               mdev->dst_offset[2] = mdev->dst_offset[1] + destination->height * buffer->video.pitch / 4;
           } else if (destination->format == DSPF_YV12) {
-               mdev->dst_cr_offset = mdev->dst_offset + destination->height * buffer->video.pitch;
-               mdev->dst_cb_offset = mdev->dst_cr_offset + destination->height * buffer->video.pitch / 4;
+               mdev->dst_offset[2] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
+               mdev->dst_offset[1] = mdev->dst_offset[2] + destination->height * buffer->video.pitch / 4;
+          } else if (destination->format == DSPF_NV12 || destination->format == DSPF_NV21) {
+               mdev->dst_offset[1] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
           }
      }
 
-     mga_waitfifo( mdrv, mdev, 3 );
-
-     if (mdev->old_matrox)
-          mga_out32( mmio, mdev->dst_offset, YDSTORG );
-     else
-          mga_out32( mmio, mdev->dst_offset, DSTORG );
+     mga_waitfifo( mdrv, mdev, depth_buffer ? 4 : 3 );
 
+     mga_out32( mmio, mdev->dst_offset[0], mdev->old_matrox ? YDSTORG : DSTORG );
      mga_out32( mmio, mdev->dst_pitch, PITCH );
 
-     switch (destination->format) {
+     if (depth_buffer)
+          mga_out32( mmio, depth_buffer->video.offset, ZORG );
+
+     switch (buffer->format) {
           case DSPF_A8:
+          case DSPF_ALUT44:
+          case DSPF_LUT8:
           case DSPF_RGB332:
                mga_out32( mmio, PW8, MACCESS );
                break;
           case DSPF_ARGB1555:
                mga_out32( mmio, PW16 | DIT555, MACCESS );
                break;
+          case DSPF_ARGB4444:
           case DSPF_RGB16:
                mga_out32( mmio, PW16, MACCESS );
                break;
@@ -104,10 +116,16 @@ void matrox_set_destination( MatroxDriverData *mdrv,
                break;
           case DSPF_I420:
           case DSPF_YV12:
+          case DSPF_NV12:
+          case DSPF_NV21:
                mga_out32( mmio, PW8 | BYPASS332 | NODITHER, MACCESS );
                break;
+          case DSPF_YUY2:
+          case DSPF_UYVY:
+               mga_out32( mmio, PW32 | NODITHER, MACCESS );
+               break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 }
@@ -121,9 +139,9 @@ void matrox_set_clip( MatroxDriverData *mdrv,
      mga_waitfifo( mdrv, mdev, 3 );
 
      if (mdev->old_matrox) {
-          mga_out32( mmio, (mdev->dst_offset +
+          mga_out32( mmio, (mdev->dst_offset[0] +
                             mdev->dst_pitch * clip->y1) & 0xFFFFFF, YTOP );
-          mga_out32( mmio, (mdev->dst_offset +
+          mga_out32( mmio, (mdev->dst_offset[0] +
                             mdev->dst_pitch * clip->y2) & 0xFFFFFF, YBOT );
      }
      else {
@@ -131,8 +149,10 @@ void matrox_set_clip( MatroxDriverData *mdrv,
           mga_out32( mmio, (mdev->dst_pitch * clip->y2) & 0xFFFFFF, YBOT );
      }
 
-     mga_out32( mmio, ((clip->x2 & 0x0FFF) << 16) |
-                       (clip->x1 & 0x0FFF), CXBNDRY );
+     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
+          mga_out32( mmio, ((clip->x2/2 & 0x0FFF) << 16) | (clip->x1/2 & 0x0FFF), CXBNDRY );
+     else
+          mga_out32( mmio, ((clip->x2 & 0x0FFF) << 16) | (clip->x1 & 0x0FFF), CXBNDRY );
 }
 
 void matrox_validate_Color( MatroxDriverData *mdrv,
@@ -162,11 +182,22 @@ void matrox_validate_color( MatroxDriverData *mdrv,
      volatile __u8 *mmio = mdrv->mmio_base;
 
      __u32 color;
+     __u8  cb, cr;
 
      if (MGA_IS_VALID( m_color ))
           return;
 
      switch (state->destination->format) {
+          case DSPF_ALUT44:
+               color = (state->color.a & 0xF0) | state->color_index;
+               color |= color << 8;
+               color |= color << 16;
+               break;
+          case DSPF_LUT8:
+               color = state->color_index;
+               color |= color << 8;
+               color |= color << 16;
+               break;
           case DSPF_RGB332:
                color = PIXEL_RGB332( state->color.r,
                                      state->color.g,
@@ -174,6 +205,13 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                color |= color << 8;
                color |= color << 16;
                break;
+          case DSPF_ARGB4444:
+               color = PIXEL_ARGB4444( state->color.a,
+                                       state->color.r,
+                                       state->color.g,
+                                       state->color.b );
+               color |= color << 16;
+               break;
           case DSPF_ARGB1555:
                color = PIXEL_ARGB1555( state->color.a,
                                        state->color.r,
@@ -209,8 +247,54 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                color |= color << 8;
                color |= color << 16;
                break;
+          case DSPF_I420:
+          case DSPF_YV12:
+               RGB_TO_YCBCR( state->color.r,
+                             state->color.g,
+                             state->color.b,
+                             color, cb, cr );
+               color |= color << 8;
+               color |= color << 16;
+               mdev->color[0] = color;
+               mdev->color[1] = (cb << 24) | (cb << 16) | (cb << 8) | cb;
+               mdev->color[2] = (cr << 24) | (cr << 16) | (cr << 8) | cr;
+               break;
+          case DSPF_NV12:
+               RGB_TO_YCBCR( state->color.r,
+                             state->color.g,
+                             state->color.b,
+                             color, cb, cr );
+               color |= color << 8;
+               color |= color << 16;
+               mdev->color[0] = color;
+               mdev->color[1] = (cr << 24) | (cb << 16) | (cr << 8) | cb;
+               break;
+          case DSPF_NV21:
+               RGB_TO_YCBCR( state->color.r,
+                             state->color.g,
+                             state->color.b,
+                             color, cb, cr );
+               color |= color << 8;
+               color |= color << 16;
+               mdev->color[0] = color;
+               mdev->color[1] = (cb << 24) | (cr << 16) | (cb << 8) | cr;
+               break;
+          case DSPF_YUY2:
+               RGB_TO_YCBCR( state->color.r,
+                             state->color.g,
+                             state->color.b,
+                             color, cb, cr );
+               color = PIXEL_YUY2( color, cb, cr );
+               break;
+          case DSPF_UYVY:
+               RGB_TO_YCBCR( state->color.r,
+                             state->color.g,
+                             state->color.b,
+                             color, cb, cr );
+               color = PIXEL_UYVY( color, cb, cr );
+               break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                return;
      }
 
@@ -259,7 +343,7 @@ void matrox_validate_drawBlend( MatroxDriverData *mdrv,
 
      if (MGA_IS_VALID( m_drawBlend ))
           return;
-     
+
      alphactrl = matroxSourceBlend[state->src_blend - 1] |
                  matroxDestBlend  [state->dst_blend - 1] | DIFFUSEDALPHA;
 
@@ -298,7 +382,7 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
      {
           alphactrl = matroxSourceBlend[state->src_blend - 1] |
                       matroxDestBlend  [state->dst_blend - 1];
-          
+
           if (state->source->format == DSPF_RGB32) {
                alphactrl |= DIFFUSEDALPHA;
 
@@ -309,7 +393,7 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
           }
           else
                alphactrl |= matroxAlphaSelect [state->blittingflags & 3];
-          
+
           if (state->dst_blend == DSBF_ZERO)
                alphactrl |= ALPHACHANNEL;
           else
@@ -317,7 +401,7 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
      }
      else {
           alphactrl = SRC_ONE | ALPHACHANNEL;
-          
+
           if (state->source->format == DSPF_RGB32) {
                alphactrl |= DIFFUSEDALPHA;
 
@@ -333,6 +417,41 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
      MGA_INVALIDATE( m_drawBlend );
 }
 
+static void matrox_tlutload( MatroxDriverData *mdrv,
+                             MatroxDeviceData *mdev,
+                             CorePalette      *palette )
+{
+     volatile __u8  *mmio = mdrv->mmio_base;
+     volatile __u16 *dst  = dfb_gfxcard_memory_virtual( NULL, mdev->tlut_offset );
+     int             i;
+
+     for (i = 0; i < palette->num_entries; i++)
+          *dst++ = PIXEL_RGB16( palette->entries[i].r,
+                                palette->entries[i].g,
+                                palette->entries[i].b );
+
+     mga_waitfifo( mdrv, mdev, 9 );
+     mga_out32( mmio, BLTMOD_BU32RGB | BOP_COPY | SHFTZERO |
+                SGNZERO | LINEAR | ATYPE_RSTR | OP_BITBLT, DWGCTL );
+     mga_out32( mmio, 1024, PITCH );
+     if (mdev->old_matrox) {
+          mga_out32( mmio, mdev->tlut_offset / 2, AR3 );
+          mga_out32( mmio, palette->num_entries, AR0 );
+          mga_out32( mmio, 0, YDSTORG );
+     }
+     else {
+          mga_out32( mmio, 0, AR3 );
+          mga_out32( mmio, palette->num_entries, AR0 );
+          mga_out32( mmio, mdev->fb.offset + mdev->tlut_offset, SRCORG );
+          mga_out32( mmio, 0, DSTORG );
+
+          MGA_INVALIDATE( m_source );
+     }
+     mga_out32( mmio, 0, FXBNDRY );
+     mga_out32( mmio, PW16 | TLUTLOAD, MACCESS );
+     mga_out32( mmio, palette->num_entries, YDSTLEN | EXECUTE );
+}
+
 void matrox_validate_Source( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CardState        *state )
@@ -349,31 +468,51 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
      mdev->src_pitch = buffer->video.pitch / bytes_per_pixel;
      mdev->field     = surface->field;
 
-     mdev->src_offset = dfb_gfxcard_memory_physical( NULL, buffer->video.offset ) & 0x1FFFFFF;
+     D_ASSERT( mdev->src_pitch % 32 == 0 );
+
+     D_ASSERT( buffer->video.offset % 32 == 0 );
+
+     mdev->src_offset[0] = mdev->fb.offset + buffer->video.offset;
      if (surface->format == DSPF_I420) {
-          mdev->src_cb_offset = mdev->src_offset + surface->height * buffer->video.pitch;
-          mdev->src_cr_offset = mdev->src_cb_offset + surface->height * buffer->video.pitch / 4;
+          mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
+          mdev->src_offset[2] = mdev->src_offset[1] + surface->height * buffer->video.pitch / 4;
      } else if (surface->format == DSPF_YV12) {
-          mdev->src_cr_offset = mdev->src_offset + surface->height * buffer->video.pitch;
-          mdev->src_cb_offset = mdev->src_cr_offset + surface->height * buffer->video.pitch / 4;
+          mdev->src_offset[2] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
+          mdev->src_offset[1] = mdev->src_offset[2] + surface->height * buffer->video.pitch / 4;
+     } else if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
+          mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
      }
 
      mdev->w = surface->width;
      mdev->h = surface->height;
+
+     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY) {
+          mdev->w /= 2;
+          mdev->src_pitch /= 2;
+     }
+
      if (mdev->blit_deinterlace) {
           mdev->h /= 2;
           if (surface->caps & DSCAPS_SEPARATED) {
                if (mdev->field) {
-                    mdev->src_offset += mdev->h * buffer->video.pitch;
-                    mdev->src_cb_offset += mdev->h * buffer->video.pitch / 4;
-                    mdev->src_cr_offset += mdev->h * buffer->video.pitch / 4;
+                    mdev->src_offset[0] += mdev->h * buffer->video.pitch;
+                    if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
+                         mdev->src_offset[1] += mdev->h * buffer->video.pitch / 2;
+                    } else {
+                         mdev->src_offset[1] += mdev->h * buffer->video.pitch / 4;
+                         mdev->src_offset[2] += mdev->h * buffer->video.pitch / 4;
+                    }
                }
           } else {
                mdev->src_pitch *= 2;
                if (mdev->field) {
-                    mdev->src_offset += buffer->video.pitch;
-                    mdev->src_cb_offset += buffer->video.pitch / 2;
-                    mdev->src_cr_offset += buffer->video.pitch / 2;
+                    mdev->src_offset[0] += buffer->video.pitch;
+                    if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
+                         mdev->src_offset[1] += buffer->video.pitch;
+                    } else {
+                         mdev->src_offset[1] += buffer->video.pitch / 2;
+                         mdev->src_offset[2] += buffer->video.pitch / 2;
+                    }
                }
           }
      }
@@ -387,16 +526,21 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
 
      switch (surface->format) {
           case DSPF_YUY2:
-               texctl |= TW422;
+               texctl |= (mdev->dst_format == DSPF_YUY2) ? TW32 : TW422;
                break;
           case DSPF_UYVY:
-               texctl |= TW422UYVY;
+               texctl |= (mdev->dst_format == DSPF_UYVY) ? TW32 : TW422UYVY;
                break;
           case DSPF_I420:
           case DSPF_YV12:
+          case DSPF_NV12:
+          case DSPF_NV21:
           case DSPF_A8:
                texctl |= TW8A;
                break;
+          case DSPF_ARGB4444:
+               texctl |= TW12;
+               break;
           case DSPF_ARGB1555:
                texctl |= TW15;
                break;
@@ -407,12 +551,25 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
           case DSPF_ARGB:
                texctl |= TW32;
                break;
+          case DSPF_LUT8:
+               matrox_tlutload( mdrv, mdev, surface->palette );
+               texctl |= TW8;
+               state->modified |= SMF_DESTINATION;
+               break;
+          case DSPF_RGB332:
+               matrox_tlutload( mdrv, mdev, mdev->rgb332_palette );
+               texctl |= TW8;
+               state->modified |= SMF_DESTINATION;
+               break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
-     texctl |= CLAMPUV | ((mdev->src_pitch&0x7ff)<<9) | PITCHEXT | NOPERSPECTIVE;
+     texctl |= ((mdev->src_pitch << 9) & TPITCHEXT) | TPITCHLIN;
+
+     if (1 << mdev->w2 != mdev->w  ||  1 << mdev->h2 != mdev->h)
+          texctl |= CLAMPUV;
 
      if (state->blittingflags & DSBLIT_COLORIZE)
           texctl |= TMODULATE;
@@ -430,11 +587,18 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
      mdev->texctl  = texctl;
 
      mga_waitfifo( mdrv, mdev, 5 );
-     mga_out32( mmio, texctl, TEXCTL );
+     mga_out32( mmio, texctl,  TEXCTL );
      mga_out32( mmio, texctl2, TEXCTL2 );
-     mga_out32( mmio, (mdev->w-1)<<18 | ((8-mdev->w2)&63)<<9 | mdev->w2, TEXWIDTH );
-     mga_out32( mmio, (mdev->h-1)<<18 | ((8-mdev->h2)&63)<<9 | mdev->h2, TEXHEIGHT );
-     mga_out32( mmio, mdev->src_offset, TEXORG );
+
+     mga_out32( mmio, ( (((__u32)(mdev->w - 1) & 0x7ff) << 18) |
+                        (((__u32)(4 - mdev->w2) & 0x3f) <<  9) |
+                        (((__u32)(mdev->w2 + 4) & 0x3f)      )  ), TEXWIDTH );
+
+     mga_out32( mmio, ( (((__u32)(mdev->h - 1) & 0x7ff) << 18) |
+                        (((__u32)(4 - mdev->h2) & 0x3f) <<  9) |
+                        (((__u32)(mdev->h2 + 4) & 0x3f)      )  ), TEXHEIGHT );
+
+     mga_out32( mmio, mdev->src_offset[0], TEXORG );
 
      MGA_VALIDATE( m_Source );
 }
@@ -453,26 +617,28 @@ void matrox_validate_source( MatroxDriverData *mdrv,
 
      mdev->src_pitch = buffer->video.pitch / bytes_per_pixel;
 
+     D_ASSERT( mdev->src_pitch % 32 == 0 );
+
+     D_ASSERT( buffer->video.offset % 64 == 0 );
+
+     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
+          mdev->src_pitch /= 2;
+
      if (mdev->old_matrox) {
-          mdev->src_offset = buffer->video.offset / bytes_per_pixel;
-          if (surface->format == DSPF_I420) {
-               mdev->src_cb_offset = mdev->src_offset + surface->height * mdev->src_pitch;
-               mdev->src_cr_offset = mdev->src_cb_offset + surface->height * mdev->src_pitch / 4;
-          } else if (surface->format == DSPF_YV12) {
-               mdev->src_cr_offset = mdev->src_offset + surface->height * mdev->src_pitch;
-               mdev->src_cb_offset = mdev->src_cr_offset + surface->height * mdev->src_pitch / 4;
-          }
+          mdev->src_offset[0] = buffer->video.offset / bytes_per_pixel;
      } else {
-          mdev->src_offset = dfb_gfxcard_memory_physical( NULL, buffer->video.offset ) & 0x1FFFFFF;
+          mdev->src_offset[0] = mdev->fb.offset + buffer->video.offset;
           if (surface->format == DSPF_I420) {
-               mdev->src_cb_offset = mdev->src_offset + surface->height * buffer->video.pitch;
-               mdev->src_cr_offset = mdev->src_cb_offset + surface->height * buffer->video.pitch / 4;
+               mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
+               mdev->src_offset[2] = mdev->src_offset[1] + surface->height * buffer->video.pitch / 4;
           } else if (surface->format == DSPF_YV12) {
-               mdev->src_cr_offset = mdev->src_offset + surface->height * buffer->video.pitch;
-               mdev->src_cb_offset = mdev->src_cr_offset + surface->height * buffer->video.pitch / 4;
+               mdev->src_offset[2] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
+               mdev->src_offset[1] = mdev->src_offset[2] + surface->height * buffer->video.pitch / 4;
+          } else if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
+               mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
           }
           mga_waitfifo( mdrv, mdev, 1 );
-          mga_out32( mmio, mdev->src_offset, SRCORG );
+          mga_out32( mmio, mdev->src_offset[0], SRCORG );
      }
 
      MGA_VALIDATE( m_source );
@@ -537,7 +703,7 @@ void matrox_validate_srckey( MatroxDriverData *mdrv,
                mga_out32( mmio, key, FCOL );
                break;
           default:
-               BUG( "unexpected bytes per pixel" );
+               D_BUG( "unexpected bytes per pixel" );
                break;
      }
 
