@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,6 +35,8 @@
 
 #include <termios.h>
 
+#include <sys/utsname.h>
+
 #include <directfb.h>
 
 #include <core/coredefs.h>
@@ -41,10 +44,14 @@
 
 #include <core/input.h>
 #include <core/system.h>
-#include <core/thread.h>
 
 #include <misc/conf.h>
-#include <misc/mem.h>
+
+#include <direct/debug.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
+#include <direct/thread.h>
+#include <direct/util.h>
 
 #include <core/input_driver.h>
 
@@ -71,13 +78,13 @@ DFB_INPUT_DRIVER( ps2mouse )
 #define PS2_ID_PS2       0
 #define PS2_ID_IMPS2     3
 
-static char *devname[2] = { "/dev/psaux", "/dev/input/mice" };
-static char *devlist[2] = { NULL, NULL };
+static int         ndev;
+static const char *devlist[8];
 
 typedef struct {
      int            fd;
-     InputDevice   *device;
-     CoreThread    *thread;
+     CoreInputDevice   *device;
+     DirectThread  *thread;
 
      int            mouseId;
      int            packetLength;
@@ -135,7 +142,7 @@ ps2mouse_motion_realize( PS2MouseData *data )
 }
 
 static void*
-ps2mouseEventThread( CoreThread *thread, void *driver_data )
+ps2mouseEventThread( DirectThread *thread, void *driver_data )
 {
      PS2MouseData *data  = (PS2MouseData*) driver_data;
 
@@ -151,7 +158,7 @@ ps2mouseEventThread( CoreThread *thread, void *driver_data )
      while ( (readlen = read(data->fd, buf, 256)) > 0 ) {
           int i;
 
-          dfb_thread_testcancel( thread );
+          direct_thread_testcancel( thread );
 
           for ( i = 0; i < readlen; i++ ) {
 
@@ -231,7 +238,7 @@ ps2mouseEventThread( CoreThread *thread, void *driver_data )
      }
 
      if ( readlen <= 0 && errno != EINTR )
-          PERRORMSG ("psmouse thread died\n");
+          D_PERROR ("psmouse thread died\n");
 
      return NULL;
 }
@@ -253,7 +260,7 @@ ps2WriteChar( int fd, unsigned char c, bool verbose )
 
      if ( select(fd+1, &fds, NULL, NULL, &tv) == 0 ) {
           if ( verbose )
-               ERRORMSG( "DirectFB/PS2Mouse: timeout waiting for ack!!\n" );
+               D_ERROR( "DirectFB/PS2Mouse: timeout waiting for ack!!\n" );
           return -1;
      }
 
@@ -289,13 +296,13 @@ ps2Write( int fd, const unsigned char *data, size_t len, bool verbose)
      for ( i = 0; i < len; i++ ) {
           if ( ps2WriteChar(fd, data[i], verbose) < 0 ) {
                if ( verbose )
-                    ERRORMSG( "DirectFB/PS2Mouse: error @byte %i\n", i );
+                    D_ERROR( "DirectFB/PS2Mouse: error @byte %i\n", i );
                error++;
           }
      }
 
      if ( error && verbose )
-          ERRORMSG( "DirectFB/PS2Mouse: missed %i ack's!\n", error);
+          D_ERROR( "DirectFB/PS2Mouse: missed %i ack's!\n", error);
 
      return( error );
 }
@@ -326,13 +333,13 @@ init_ps2( int fd, bool verbose )
           FD_ZERO( &fds );
           FD_SET( fd, &fds );
 
-          if (select( fd+1, &fds, NULL, NULL, &tv )) 
+          if (select( fd+1, &fds, NULL, NULL, &tv ))
                read( fd, buf, sizeof(buf) );
           else
                break;
 
           if (! --count) {
-               ERRORMSG( "DirectFB/PS2Mouse: "
+               D_ERROR( "DirectFB/PS2Mouse: "
                          "PS/2 mouse keeps sending data, "
                          "initialization failed\n" );
                return -1;
@@ -343,7 +350,7 @@ init_ps2( int fd, bool verbose )
      /* Do a basic init in case the mouse is confused */
      if (ps2Write( fd, basic_init, sizeof (basic_init), verbose ) != 0) {
           if (verbose)
-               ERRORMSG( "DirectFB/PS2Mouse: PS/2 mouse failed init\n" );
+               D_ERROR( "DirectFB/PS2Mouse: PS/2 mouse failed init\n" );
           return -1;
      }
 
@@ -351,7 +358,7 @@ init_ps2( int fd, bool verbose )
 
      if (ps2Write(fd, imps2_init, sizeof (imps2_init), verbose) != 0) {
           if (verbose)
-               ERRORMSG ("DirectFB/PS2Mouse: mouse failed IMPS/2 init\n");
+               D_ERROR ("DirectFB/PS2Mouse: mouse failed IMPS/2 init\n");
           return -2;
      }
 
@@ -364,32 +371,61 @@ init_ps2( int fd, bool verbose )
      return mouseId;
 }
 
+/**************************************************************************************************/
 
-/* exported symbols */
+static void
+check_devices( const char *devnames[], int num )
+{
+     int i, fd;
+
+     for (i=0; i<num; i++) {
+          if ((fd = open( devnames[i], O_RDWR | O_SYNC )) < 0)
+               continue;
+
+          if (init_ps2( fd, false ) < 0) {
+               close( fd );
+               continue;
+          }
+
+          devlist[ndev++] = devnames[i];
+
+          close( fd );
+
+          break;
+     }
+}
+
+#define CHECK_DEVICES(n)  check_devices( n, D_ARRAY_SIZE(n) )
+
+/**************************************************************************************************/
+
+static const char *dev_psaux[2] = { "/dev/psaux", "/dev/misc/psaux" };
+static const char *dev_input[1] = { "/dev/input/mice" };
+
+/**************************************************************************************************/
 
 static int
 driver_get_available()
 {
-     int fd;
-     int n_dev = 0;
-     int i;
+     struct utsname uts;
+     bool           check_psaux = true;
+     bool           check_input = true;
 
      if (dfb_system_type() != CORE_FBDEV)
           return 0;
 
-     for (i = 0; i < (int)(sizeof(devname) / sizeof(devname[0])); i++) {
-          fd = open( devname[i], O_RDWR | O_SYNC );
+     if (uname( &uts ) < 0)
+          D_PERROR( "DirectFB/PS2Mouse: uname() failed!\n" );
+     else if (!strncmp( uts.release, "2.6.", 4 ) || !strncmp( uts.release, "2.5.", 4 ))
+          check_psaux = false;
 
-          if (fd >= 0) {
-               if (init_ps2( fd, false ) >= 0) {
-                    devlist[n_dev] = devname[i];
-                    n_dev++;
-               }
-               close( fd );
-          }
-     }
+     if (check_psaux)
+          CHECK_DEVICES( dev_psaux );
 
-     return n_dev;
+     if (check_input)
+          CHECK_DEVICES( dev_input );
+
+     return ndev;
 }
 
 static void
@@ -401,14 +437,14 @@ driver_get_info( InputDriverInfo *info )
 
      snprintf( info->vendor,
                DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "Convergence GmbH" );
 
-     info->version.major = 0;
-     info->version.minor = 9;
+     info->version.major = 1;
+     info->version.minor = 0;
 }
 
 static DFBResult
-driver_open_device( InputDevice      *device,
+driver_open_device( CoreInputDevice  *device,
                     unsigned int      number,
                     InputDeviceInfo  *info,
                     void            **driver_data )
@@ -420,14 +456,14 @@ driver_open_device( InputDevice      *device,
 
      fd = open( devlist[number], O_RDWR | O_SYNC | O_EXCL );
      if (fd < 0) {
-          PERRORMSG( "DirectFB/PS2Mouse: failed opening `%s' !\n",
+          D_PERROR( "DirectFB/PS2Mouse: failed opening `%s' !\n",
                      devlist[number] );
           close( fd );
           return DFB_INIT;
      }
 
      if ((mouseId = init_ps2(fd, true)) < 0) {
-          PERRORMSG( "DirectFB/PS2Mouse: could not initialize mouse on `%s'!\n",
+          D_PERROR( "DirectFB/PS2Mouse: could not initialize mouse on `%s'!\n",
                      devlist[number] );
           close( fd );
           return DFB_INIT;
@@ -447,7 +483,7 @@ driver_open_device( InputDevice      *device,
      info->desc.max_button = DIBI_MIDDLE;     /* TODO: probe!? */
 
      /* allocate and fill private data */
-     data = DFBCALLOC( 1, sizeof(PS2MouseData) );
+     data = D_CALLOC( 1, sizeof(PS2MouseData) );
 
      data->fd           = fd;
      data->device       = device;
@@ -455,11 +491,11 @@ driver_open_device( InputDevice      *device,
      data->packetLength = (mouseId == PS2_ID_IMPS2) ? 4 : 3;
 
      /* start input thread */
-     data->thread = dfb_thread_create( CTT_INPUT, ps2mouseEventThread, data );
+     data->thread = direct_thread_create( DTT_INPUT, ps2mouseEventThread, data, "PS/2 Input" );
 
      /* set private data pointer */
      *driver_data = data;
-     
+
      return DFB_OK;
 }
 
@@ -467,7 +503,7 @@ driver_open_device( InputDevice      *device,
  * Fetch one entry from the device's keymap if supported.
  */
 static DFBResult
-driver_get_keymap_entry( InputDevice               *device,
+driver_get_keymap_entry( CoreInputDevice               *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
@@ -480,13 +516,13 @@ driver_close_device( void *driver_data )
      PS2MouseData *data = (PS2MouseData*) driver_data;
 
      /* stop input thread */
-     dfb_thread_cancel( data->thread );
-     dfb_thread_join( data->thread );
-     dfb_thread_destroy( data->thread );
+     direct_thread_cancel( data->thread );
+     direct_thread_join( data->thread );
+     direct_thread_destroy( data->thread );
 
      /* close device */
      close( data->fd );
 
      /* free private data */
-     DFBFREE( data );
+     D_FREE( data );
 }
