@@ -2,19 +2,19 @@
 
    Driver for Wolfson WM9705/WM9712 touchscreen controllers as
    found in the Dell Axim and Toshiba e740.
-   
+
    liam.girdwood@wolfsonmicro.com
-   
-   NOTE: Please make sure that pressure measurement is enable in the 
-         kernel driver. (i.e. pil != 0) Please read 
+
+   NOTE: Please make sure that pressure measurement is enable in the
+         kernel driver. (i.e. pil != 0) Please read
 		  wolfson-touchscreen.txt in your kernel documentation for
 		  details.
-   
+
    Based on the h3600_ts.c driver by convergence GmbH.
 
    (c) Copyright 2000-2002  convergence integrated media GmbH.
    (c) Copyright 2002       convergence GmbH.
-   
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
@@ -54,10 +54,13 @@
 #include <core/coretypes.h>
 
 #include <core/input.h>
-#include <core/thread.h>
 
 #include <misc/conf.h>
-#include <misc/mem.h>
+
+#include <direct/debug.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
+#include <direct/thread.h>
 
 #include <core/input_driver.h>
 
@@ -65,14 +68,23 @@
 DFB_INPUT_DRIVER( wm97xx_ts )
 
 typedef struct {
-     InputDevice   *device;
-     CoreThread    *thread;
+     CoreInputDevice   *device;
+     DirectThread  *thread;
 
      int            fd;
 } WM97xxTSData;
 
+/* I copied this from the kernel driver since there is no include file */
+typedef struct {
+     __u16             pressure;
+     __u16             x;
+     __u16             y;
+     __u16             pad;
+     struct timeval  stamp;
+} TS_EVENT;
+
 static void *
-wm97xxtsEventThread( CoreThread *thread, void *driver_data )
+wm97xxtsEventThread( DirectThread *thread, void *driver_data )
 {
      WM97xxTSData *data = (WM97xxTSData*) driver_data;
 
@@ -89,7 +101,7 @@ wm97xxtsEventThread( CoreThread *thread, void *driver_data )
      {
           DFBInputEvent evt;
 
-          dfb_thread_testcancel( thread );
+          direct_thread_testcancel( thread );
 
           if (readlen < 1)
                continue;
@@ -132,7 +144,7 @@ wm97xxtsEventThread( CoreThread *thread, void *driver_data )
      }
 
      if (readlen <= 0)
-          PERRORMSG ("WM97xx Touchscreen thread died\n");
+          D_PERROR ("WM97xx Touchscreen thread died\n");
 
      return NULL;
 }
@@ -169,7 +181,7 @@ driver_get_info( InputDriverInfo *info )
 }
 
 static DFBResult
-driver_open_device( InputDevice      *device,
+driver_open_device( CoreInputDevice      *device,
                     unsigned int      number,
                     InputDeviceInfo  *info,
                     void            **driver_data )
@@ -180,7 +192,7 @@ driver_open_device( InputDevice      *device,
      /* open device */
      fd = open( "/dev/touchscreen/wm97xx", O_RDONLY | O_NOCTTY );
      if (fd < 0) {
-          PERRORMSG( "DirectFB/WM97xx: Error opening `/dev/touchscreen/wm97xx'!\n" );
+          D_PERROR( "DirectFB/WM97xx: Error opening `/dev/touchscreen/wm97xx'!\n" );
           return DFB_INIT;
      }
 
@@ -199,13 +211,13 @@ driver_open_device( InputDevice      *device,
      info->desc.max_button = DIBI_LEFT;
 
      /* allocate and fill private data */
-     data = DFBCALLOC( 1, sizeof(WM97xxTSData) );
+     data = D_CALLOC( 1, sizeof(WM97xxTSData) );
 
      data->fd     = fd;
      data->device = device;
 
      /* start input thread */
-     data->thread = dfb_thread_create( CTT_INPUT, wm97xxtsEventThread, data );
+     data->thread = direct_thread_create( DTT_INPUT, wm97xxtsEventThread, data, "WM97xx TS Input" );
 
      /* set private data pointer */
      *driver_data = data;
@@ -217,7 +229,7 @@ driver_open_device( InputDevice      *device,
  * Fetch one entry from the device's keymap if supported.
  */
 static DFBResult
-driver_get_keymap_entry( InputDevice               *device,
+driver_get_keymap_entry( CoreInputDevice               *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
@@ -230,14 +242,14 @@ driver_close_device( void *driver_data )
      WM97xxTSData *data = (WM97xxTSData*) driver_data;
 
      /* stop input thread */
-     dfb_thread_cancel( data->thread );
-     dfb_thread_join( data->thread );
-     dfb_thread_destroy( data->thread );
+     direct_thread_cancel( data->thread );
+     direct_thread_join( data->thread );
+     direct_thread_destroy( data->thread );
 
      /* close device */
      if (close( data->fd ) < 0)
-          PERRORMSG( "DirectFB/WM97xx: Error closing `/dev/touchscreen/wm97xx'!\n" );
+          D_PERROR( "DirectFB/WM97xx: Error closing `/dev/touchscreen/wm97xx'!\n" );
 
      /* free private data */
-     DFBFREE( data );
+     D_FREE( data );
 }
