@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,12 +25,14 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
-
-#include <freetype/ftglyph.h>
+#include <ft2build.h>
+#include FT_GLYPH_H
 
 #include <directfb.h>
 
@@ -42,9 +45,12 @@
 
 #include <media/idirectfbfont.h>
 
-#include <misc/mem.h>
-#include <misc/memcpy.h>
-#include <misc/tree.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
+#include <misc/conf.h>
 #include <misc/util.h>
 
 
@@ -53,16 +59,17 @@ Probe( IDirectFBFont_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
+           CoreDFB            *core,
            const char         *filename,
            DFBFontDescription *desc );
 
-#include <interface_implementation.h>
+#include <direct/interface_implementation.h>
 
-DFB_INTERFACE_IMPLEMENTATION( IDirectFBFont, FT2 )
+DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBFont, FT2 )
 
 static FT_Library      library           = NULL;
 static int             library_ref_count = 0;
-static pthread_mutex_t library_mutex     = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t library_mutex     = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
 
 #define KERNING_CACHE_MIN   32
 #define KERNING_CACHE_MAX  127
@@ -83,15 +90,15 @@ typedef struct {
      int          fixed_advance;
 } FT2ImplData;
 
+typedef struct {
+     char x;
+     char y;
+} KerningCacheEntry;
 
 typedef struct {
      FT2ImplData base;
 
-     struct {
-          int  x   : 16;
-          int  y   : 15;
-          bool got : 1;
-     } kerning[KERNING_CACHE_SIZE][KERNING_CACHE_SIZE];
+     KerningCacheEntry kerning[KERNING_CACHE_SIZE][KERNING_CACHE_SIZE];
 } FT2ImplKerningData;
 
 
@@ -112,7 +119,7 @@ render_glyph( CoreFont      *thiz,
      FT2ImplData *data = (FT2ImplData*) thiz->impl_data;
 
      pthread_mutex_lock ( &library_mutex );
-     
+
      face = data->face;
 
      if (data->disable_charmap)
@@ -124,18 +131,18 @@ render_glyph( CoreFont      *thiz,
      load_flags |= FT_LOAD_RENDER;
 
      if ((err = FT_Load_Glyph( face, index, load_flags ))) {
-          HEAVYDEBUGMSG( "DirectFB/FontFT2: "
+          D_HEAVYDEBUG( "DirectFB/FontFT2: "
                          "Could not render glyph for character #%d!\n", glyph );
           pthread_mutex_unlock ( &library_mutex );
           return DFB_FAILURE;
      }
 
      pthread_mutex_unlock ( &library_mutex );
-     
+
      err = dfb_surface_soft_lock( surface, DSLF_WRITE,
                                   &dst, &pitch, 0 );
      if (err) {
-          ERRORMSG( "DirectB/FontFT2: Unable to lock surface!\n" );
+          D_ERROR( "DirectB/FontFT2: Unable to lock surface!\n" );
           return err;
      }
 
@@ -151,11 +158,12 @@ render_glyph( CoreFont      *thiz,
      info->top  = thiz->ascender - face->glyph->bitmap_top;
 
      src = face->glyph->bitmap.buffer;
-     dst += thiz->next_x * DFB_BYTES_PER_PIXEL(surface->format);
+     dst += DFB_BYTES_PER_LINE(surface->format, thiz->next_x);
 
      for (y=0; y < info->height; y++) {
-          int    i;
+          int    i, j, n;
           __u8  *dst8  = dst;
+          __u16 *dst16 = dst;
           __u32 *dst32 = dst;
 
           switch (face->glyph->bitmap.pixel_mode) {
@@ -165,8 +173,34 @@ render_glyph( CoreFont      *thiz,
                               for (i=0; i<info->width; i++)
                                    dst32[i] = (src[i] << 24) | 0xFFFFFF;
                               break;
+                         case DSPF_AiRGB:
+                              for (i=0; i<info->width; i++)
+                                   dst32[i] = ((src[i] ^ 0xFF) << 24) | 0xFFFFFF;
+                              break;
+                         case DSPF_ARGB4444:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (src[i] << 8) | 0xFFF;
+                              break;
+                         case DSPF_ARGB2554:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (src[i] << 8) | 0x3FFF;
+                              break;
+                         case DSPF_ARGB1555:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (src[i] << 8) | 0x7FFF;
+                              break;
                          case DSPF_A8:
-                              dfb_memcpy( dst, src, info->width );
+                              direct_memcpy( dst, src, info->width );
+                              break;
+                         case DSPF_A1:
+                              for (i=0, j=0; i < info->width; ++j) {
+                                   register __u8 p = 0;
+
+                                   for (n=0; n<8 && i<info->width; ++i, ++n)
+                                        p |= (src[i] & 0x80) >> n;
+
+                                   dst8[j] = p;
+                              }
                               break;
                          default:
                               break;
@@ -180,11 +214,34 @@ render_glyph( CoreFont      *thiz,
                                    dst32[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
                                                 0xFF : 0x00) << 24) | 0xFFFFFF;
                               break;
+                         case DSPF_AiRGB:
+                              for (i=0; i<info->width; i++)
+                                   dst32[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
+                                                0x00 : 0xFF) << 24) | 0xFFFFFF;
+                              break;
+                         case DSPF_ARGB4444:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
+                                                0xF : 0x0) << 12) | 0xFFF;
+                              break;
+                         case DSPF_ARGB2554:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
+                                                0x3 : 0x0) << 14) | 0x3FFF;
+                              break;
+                         case DSPF_ARGB1555:
+                              for (i=0; i<info->width; i++)
+                                   dst16[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
+                                                0x1 : 0x0) << 15) | 0x7FFF;
+                              break;
                          case DSPF_A8:
                               for (i=0; i<info->width; i++)
                                    dst8[i] = (src[i>>3] &
                                               (1<<(7-(i%8)))) ? 0xFF : 0x00;
                               break;
+                         case DSPF_A1:
+                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(DSPF_A1, info->width) );
+                              break;
                          default:
                               break;
                     }
@@ -217,7 +274,7 @@ get_glyph_info( CoreFont      *thiz,
      FT2ImplData *data = (FT2ImplData*) thiz->impl_data;
 
      pthread_mutex_lock ( &library_mutex );
-     
+
      face = data->face;
 
      if (data->disable_charmap)
@@ -228,7 +285,7 @@ get_glyph_info( CoreFont      *thiz,
      load_flags = (FT_Int) face->generic.data;
 
      if ((err = FT_Load_Glyph( face, index, load_flags ))) {
-          HEAVYDEBUGMSG( "DirectB/FontFT2: "
+          D_HEAVYDEBUG( "DirectB/FontFT2: "
                          "Could not load glyph for character #%d!\n", glyph );
 
           pthread_mutex_unlock ( &library_mutex );
@@ -239,7 +296,7 @@ get_glyph_info( CoreFont      *thiz,
      if (face->glyph->format != ft_glyph_format_bitmap) {
           err = FT_Render_Glyph( face->glyph, ft_render_mode_normal );
           if (err) {
-               ERRORMSG( "DirectFB/FontFT2: Could not "
+               D_ERROR( "DirectFB/FontFT2: Could not "
                          "render glyph for character #%d!\n", glyph );
 
                pthread_mutex_unlock ( &library_mutex );
@@ -249,7 +306,7 @@ get_glyph_info( CoreFont      *thiz,
      }
 
      pthread_mutex_unlock ( &library_mutex );
-     
+
      info->width   = face->glyph->bitmap.width;
      info->height  = face->glyph->bitmap.rows;
      info->advance = data->fixed_advance ?
@@ -271,21 +328,28 @@ get_kerning( CoreFont *thiz,
      FT_UInt    current_index;
 
      FT2ImplKerningData *data = (FT2ImplKerningData*) thiz->impl_data;
+     KerningCacheEntry *cache = NULL;
+
+     D_ASSUME( (kern_x != NULL) || (kern_y != NULL) );
 
      /*
       * Use cached values if characters are in the
       * cachable range and the cache entry is already filled.
       */
-     if (KERNING_DO_CACHE (prev, current) &&
-         KERNING_CACHE_ENTRY (prev, current).got) {
-          *kern_x = KERNING_CACHE_ENTRY (prev, current).x;
-          *kern_y = KERNING_CACHE_ENTRY (prev, current).y;
+     if (KERNING_DO_CACHE (prev, current)) {
+          cache = &KERNING_CACHE_ENTRY (prev, current);
+
+          if (kern_x)
+               *kern_x = (int) cache->x;
+
+          if (kern_y)
+               *kern_y = (int) cache->y;
 
           return DFB_OK;
      }
 
      pthread_mutex_lock ( &library_mutex );
-     
+
      /* Get the character indices for lookup. */
      prev_index    = FT_Get_Char_Index( data->base.face, prev );
      current_index = FT_Get_Char_Index( data->base.face, current );
@@ -295,23 +359,46 @@ get_kerning( CoreFont *thiz,
                      prev_index, current_index, ft_kerning_default, &vector );
 
      pthread_mutex_unlock ( &library_mutex );
-     
-     /* Convert to integer. */
-     *kern_x = vector.x >> 6;
-     *kern_y = vector.y >> 6;
 
-     /*
-      * Fill cache entry if characters are in the cachable range.
-      */
-     if (KERNING_DO_CACHE (prev, current)) {
-          KERNING_CACHE_ENTRY (prev, current).x   = *kern_x;
-          KERNING_CACHE_ENTRY (prev, current).y   = *kern_y;
-          KERNING_CACHE_ENTRY (prev, current).got = true;
-     }
+     /* Convert to integer. */
+     if (kern_x)
+          *kern_x = vector.x >> 6;
+
+     if (kern_y)
+          *kern_y = vector.y >> 6;
 
      return DFB_OK;
 }
 
+static void
+init_kerning_cache( FT2ImplKerningData *data )
+{
+     int a, b;
+
+     for (a=KERNING_CACHE_MIN; a<=KERNING_CACHE_MAX; a++) {
+          for (b=KERNING_CACHE_MIN; b<=KERNING_CACHE_MAX; b++) {
+               FT_Vector          vector;
+               FT_UInt            prev;
+               FT_UInt            current;
+               KerningCacheEntry *cache = &KERNING_CACHE_ENTRY( a, b );
+
+               pthread_mutex_lock ( &library_mutex );
+
+               /* Get the character indices for lookup. */
+               prev    = FT_Get_Char_Index( data->base.face, a );
+               current = FT_Get_Char_Index( data->base.face, b );
+
+               /* Lookup kerning values for the character pair. */
+               FT_Get_Kerning( data->base.face,
+                               prev, current, ft_kerning_default, &vector );
+
+               pthread_mutex_unlock ( &library_mutex );
+
+               cache->x = (char) (vector.x >> 6);
+               cache->y = (char) (vector.y >> 6);
+          }
+     }
+}
 
 static DFBResult
 init_freetype( void )
@@ -321,11 +408,11 @@ init_freetype( void )
      pthread_mutex_lock ( &library_mutex );
 
      if (!library) {
-          HEAVYDEBUGMSG( "DirectFB/FontFT2: "
+          D_HEAVYDEBUG( "DirectFB/FontFT2: "
                          "Initializing the FreeType2 library.\n" );
           err = FT_Init_FreeType( &library );
           if (err) {
-               ERRORMSG( "DirectFB/FontFT2: "
+               D_ERROR( "DirectFB/FontFT2: "
                          "Initialization of the FreeType2 library failed!\n" );
                library = NULL;
                pthread_mutex_unlock( &library_mutex );
@@ -346,7 +433,7 @@ release_freetype( void )
      pthread_mutex_lock( &library_mutex );
 
      if (library && --library_ref_count == 0) {
-          HEAVYDEBUGMSG( "DirectFB/FontFT2: "
+          D_HEAVYDEBUG( "DirectFB/FontFT2: "
                          "Releasing the FreeType2 library.\n" );
           FT_Done_FreeType( library );
           library = NULL;
@@ -368,7 +455,7 @@ IDirectFBFont_FT2_Destruct( IDirectFBFont *thiz )
           FT_Done_Face( impl_data->face );
           pthread_mutex_unlock ( &library_mutex );
 
-          DFBFREE( impl_data );
+          D_FREE( impl_data );
 
           data->font->impl_data = NULL;
      }
@@ -382,7 +469,7 @@ IDirectFBFont_FT2_Destruct( IDirectFBFont *thiz )
 static DFBResult
 IDirectFBFont_FT2_Release( IDirectFBFont *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBFont)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
      if (--data->ref == 0) {
           IDirectFBFont_FT2_Destruct( thiz );
@@ -398,7 +485,7 @@ Probe( IDirectFBFont_ProbeContext *ctx )
      FT_Error err;
      FT_Face  face;
 
-     HEAVYDEBUGMSG( "DirectFB/FontFT2: Probe font `%s'.\n", ctx->filename );
+     D_HEAVYDEBUG( "DirectFB/FontFT2: Probe font `%s'.\n", ctx->filename );
 
      if (!ctx->filename)
           return DFB_UNSUPPORTED;
@@ -419,18 +506,20 @@ Probe( IDirectFBFont_ProbeContext *ctx )
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
+           CoreDFB            *core,
            const char         *filename,
            DFBFontDescription *desc )
 {
-     CoreFont    *font;
-     FT_Face      face;
-     FT_Error     err;
-     FT_Int       load_flags = FT_LOAD_DEFAULT;
-     FT2ImplData *data;
-     bool         disable_charmap = false;
-     bool         disable_kerning = false;
+     CoreFont              *font;
+     FT_Face                face;
+     FT_Error               err;
+     FT_Int                 load_flags = FT_LOAD_DEFAULT;
+     FT2ImplData           *data;
+     bool                   disable_charmap = false;
+     bool                   disable_kerning = false;
+     bool                   load_mono = false;
 
-     HEAVYDEBUGMSG( "DirectFB/FontFT2: "
+     D_HEAVYDEBUG( "DirectFB/FontFT2: "
                     "Construct font from file `%s' (index %d) at pixel size %d x %d.\n",
                     filename,
                     (desc->flags & DFDESC_INDEX)  ? desc->index  : 0,
@@ -438,7 +527,7 @@ Construct( IDirectFBFont      *thiz,
                     (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
 
      if (init_freetype() != DFB_OK) {
-          DFB_DEALLOCATE_INTERFACE( thiz );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
           return DFB_FAILURE;
      }
 
@@ -450,20 +539,23 @@ Construct( IDirectFBFont      *thiz,
      if (err) {
           switch (err) {
                case FT_Err_Unknown_File_Format:
-                    ERRORMSG( "DirectFB/FontFT2: "
+                    D_ERROR( "DirectFB/FontFT2: "
                               "Unsupported font format in file `%s'!\n", filename );
                     break;
                default:
-                    ERRORMSG( "DirectFB/FontFT2: "
+                    D_ERROR( "DirectFB/FontFT2: "
                               "Failed loading face %d from font file `%s'!\n",
                               (desc->flags & DFDESC_INDEX) ? desc->index : 0,
                               filename );
                     break;
           }
-          DFB_DEALLOCATE_INTERFACE( thiz );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
           return DFB_FAILURE;
      }
 
+     if (dfb_config->font_format == DSPF_A1 || dfb_config->font_format == DSPF_ARGB1555)
+          load_mono = true;
+
      if (desc->flags & DFDESC_ATTRIBUTES) {
           if (desc->attributes & DFFA_NOHINTING)
                load_flags |= FT_LOAD_NO_HINTING;
@@ -471,12 +563,15 @@ Construct( IDirectFBFont      *thiz,
                disable_charmap = true;
           if (desc->attributes & DFFA_NOKERNING)
                disable_kerning = true;
-
           if (desc->attributes & DFFA_MONOCHROME)
+               load_mono = true;
+     }
+
+     if (load_mono) {
 #ifdef FT_LOAD_TARGET_MONO  /* added in FreeType-2.1.3 */
-               load_flags |= FT_LOAD_TARGET_MONO;
+          load_flags |= FT_LOAD_TARGET_MONO;
 #else
-               load_flags |= FT_LOAD_MONOCHROME;
+          load_flags |= FT_LOAD_MONOCHROME;
 #endif
      }
 
@@ -489,7 +584,7 @@ Construct( IDirectFBFont      *thiz,
 
           /* ft_encoding_latin_1 has been introduced in freetype-2.1 */
           if (err) {
-               HEAVYDEBUGMSG( "DirectFB/FontFT2: "
+               D_HEAVYDEBUG( "DirectFB/FontFT2: "
                               "Couldn't select Unicode encoding, "
                               "falling back to Latin1.\n");
                pthread_mutex_lock ( &library_mutex );
@@ -501,12 +596,12 @@ Construct( IDirectFBFont      *thiz,
 
 #if 0
      if (err) {
-          ERRORMSG( "DirectFB/FontFT2: "
+          D_ERROR( "DirectFB/FontFT2: "
                     "Couldn't select a suitable encoding for face %d from font file `%s'!\n", (desc->flags & DFDESC_INDEX) ? desc->index : 0, filename );
           pthread_mutex_lock ( &library_mutex );
           FT_Done_Face( face );
           pthread_mutex_unlock ( &library_mutex );
-          DFB_DEALLOCATE_INTERFACE( thiz );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
           return DFB_FAILURE;
      }
 #endif
@@ -518,14 +613,14 @@ Construct( IDirectFBFont      *thiz,
                                     (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
           pthread_mutex_unlock ( &library_mutex );
           if (err) {
-               ERRORMSG( "DirectB/FontFT2: "
+               D_ERROR( "DirectB/FontFT2: "
                          "Could not set pixel size to %d x %d!\n",
                          (desc->flags & DFDESC_WIDTH)  ? desc->width  : 0,
                          (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
                pthread_mutex_lock ( &library_mutex );
                FT_Done_Face( face );
                pthread_mutex_unlock ( &library_mutex );
-               DFB_DEALLOCATE_INTERFACE( thiz );
+               DIRECT_DEALLOCATE_INTERFACE( thiz );
                return DFB_FAILURE;
           }
      }
@@ -533,33 +628,41 @@ Construct( IDirectFBFont      *thiz,
      face->generic.data = (void *) load_flags;
      face->generic.finalizer = NULL;
 
-     font = dfb_font_create();
+     font = dfb_font_create( core );
 
-     DFB_ASSERT( font->pixel_format == DSPF_ARGB || 
-                 font->pixel_format == DSPF_A8 );
+     D_ASSERT( font->pixel_format == DSPF_ARGB ||
+               font->pixel_format == DSPF_AiRGB ||
+               font->pixel_format == DSPF_ARGB4444 ||
+               font->pixel_format == DSPF_ARGB2554 ||
+               font->pixel_format == DSPF_ARGB1555 ||
+               font->pixel_format == DSPF_A8 ||
+               font->pixel_format == DSPF_A1 );
 
      font->ascender   = face->size->metrics.ascender >> 6;
      font->descender  = face->size->metrics.descender >> 6;
      font->height     = font->ascender + ABS(font->descender) + 1;
      font->maxadvance = face->size->metrics.max_advance >> 6;
 
-     HEAVYDEBUGMSG( "DirectFB/FontFT2: font->height = %d\n", font->height );
-     HEAVYDEBUGMSG( "DirectFB/FontFT2: font->ascender = %d\n", font->ascender );
-     HEAVYDEBUGMSG( "DirectFB/FontFT2: font->descender = %d\n",font->descender );
+     D_HEAVYDEBUG( "DirectFB/FontFT2: font->height = %d\n", font->height );
+     D_HEAVYDEBUG( "DirectFB/FontFT2: font->ascender = %d\n", font->ascender );
+     D_HEAVYDEBUG( "DirectFB/FontFT2: font->descender = %d\n",font->descender );
 
      font->GetGlyphInfo = get_glyph_info;
      font->RenderGlyph  = render_glyph;
 
      if (FT_HAS_KERNING(face) && !disable_kerning) {
           font->GetKerning = get_kerning;
-          data = DFBCALLOC( 1, sizeof(FT2ImplKerningData) );
+          data = D_CALLOC( 1, sizeof(FT2ImplKerningData) );
      }
      else
-          data = DFBCALLOC( 1, sizeof(FT2ImplData) );
+          data = D_CALLOC( 1, sizeof(FT2ImplData) );
 
      data->face            = face;
      data->disable_charmap = disable_charmap;
 
+     if (FT_HAS_KERNING(face) && !disable_kerning)
+          init_kerning_cache( (FT2ImplKerningData*) data );
+
      if (desc->flags & DFDESC_FIXEDADVANCE) {
           data->fixed_advance = desc->fixed_advance;
           font->maxadvance    = desc->fixed_advance;
