@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,6 +25,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -32,7 +35,6 @@
 
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <display/idirectfbsurface.h>
 
@@ -46,7 +48,9 @@
 
 #include <misc/gfx_util.h>
 #include <misc/util.h>
-#include <misc/mem.h>
+#include <direct/interface.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
 
 #include <jpeglib.h>
 #include <setjmp.h>
@@ -59,9 +63,9 @@ static DFBResult
 Construct( IDirectFBImageProvider *thiz,
            IDirectFBDataBuffer    *buffer );
 
-#include <interface_implementation.h>
+#include <direct/interface_implementation.h>
 
-DFB_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, JPEG )
+DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, JPEG )
 
 /*
  * private data struct of IDirectFBImageProvider_JPEG
@@ -70,6 +74,9 @@ typedef struct {
      int                  ref;      /* reference counter */
 
      IDirectFBDataBuffer *buffer;
+
+     DIRenderCallback     render_callback;
+     void                *render_callback_context;
 } IDirectFBImageProvider_JPEG_data;
 
 static DFBResult
@@ -141,7 +148,7 @@ buffer_fill_input_buffer (j_decompress_ptr cinfo)
           src->data[0] = (JOCTET) 0xFF;
           src->data[1] = (JOCTET) JPEG_EOI;
           nbytes = 2;
-          
+
           if (ret)
                DirectFBError( "(DirectFB/ImageProvider_JPEG) GetData failed", ret );
      }
@@ -182,7 +189,7 @@ jpeg_buffer_src (j_decompress_ptr cinfo, IDirectFBDataBuffer *buffer)
                                            sizeof (buffer_source_mgr));
 
      src = (buffer_src_ptr) cinfo->src;
-     
+
      src->data = (JOCTET *)
                   cinfo->mem->alloc_small ((j_common_ptr) cinfo, JPOOL_PERMANENT,
                                            JPEG_PROG_BUF_SIZE * sizeof (JOCTET));
@@ -287,7 +294,7 @@ static DFBResult
 Construct( IDirectFBImageProvider *thiz,
            IDirectFBDataBuffer    *buffer )
 {
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_JPEG)
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_JPEG)
 
      data->ref    = 1;
      data->buffer = buffer;
@@ -313,13 +320,13 @@ IDirectFBImageProvider_JPEG_Destruct( IDirectFBImageProvider *thiz )
 
      data->buffer->Release( data->buffer );
 
-     DFB_DEALLOCATE_INTERFACE( thiz );
+     DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 static DFBResult
 IDirectFBImageProvider_JPEG_AddRef( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
 
      data->ref++;
 
@@ -329,7 +336,7 @@ IDirectFBImageProvider_JPEG_AddRef( IDirectFBImageProvider *thiz )
 static DFBResult
 IDirectFBImageProvider_JPEG_Release( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
 
      if (--data->ref == 0) {
           IDirectFBImageProvider_JPEG_Destruct( thiz );
@@ -352,7 +359,7 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
 
-     INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
 
      dst_data = (IDirectFBSurface_data*) destination->priv;
      if (!dst_data)
@@ -429,11 +436,13 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
               rect.h == (int)cinfo.output_height &&
               direct)
           {
+               int y = 0;
+
                /* image must not be scaled */
                row_ptr = dst;
 
                dst += rect.x * DFB_BYTES_PER_PIXEL(format) + rect.y * pitch;
-               
+
                while (cinfo.output_scanline < cinfo.output_height) {
                     jpeg_read_scanlines(&cinfo, buffer, 1);
                     switch (format) {
@@ -459,13 +468,23 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
                                            cinfo.output_width);
                               break;
                          default:
-                              BUG("unsupported format not filtered before");
+                              D_BUG("unsupported format not filtered before");
                               return DFB_BUG;
                     }
                     row_ptr += pitch;
+
+                    y++;
+
+                    if (data->render_callback) {
+                         DFBRectangle rect = { 0, y, cinfo.output_width, 1 };
+
+                         data->render_callback( &rect, data->render_callback_context );
+                    }
                }
           }
           else {     /* image must be scaled */
+               int y = 0;
+
                image_data = malloc(cinfo.output_width * cinfo.output_height*4);
                row_ptr = image_data;
 
@@ -473,6 +492,14 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
                     jpeg_read_scanlines(&cinfo, buffer, 1);
                     copy_line32( (__u32*)row_ptr, *buffer, cinfo.output_width);
                     row_ptr += cinfo.output_width * 4;
+
+                    y++;
+
+                    if (data->render_callback) {
+                         DFBRectangle rect = { 0, y, cinfo.output_width, 1 };
+
+                         data->render_callback( &rect, data->render_callback_context );
+                    }
                }
 
                dfb_scale_linear_32( image_data,
@@ -498,7 +525,12 @@ IDirectFBImageProvider_JPEG_SetRenderCallback( IDirectFBImageProvider *thiz,
                                                DIRenderCallback        callback,
                                                void                   *context )
 {
-     return DFB_UNIMPLEMENTED;
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_JPEG)
+
+     data->render_callback         = callback;
+     data->render_callback_context = context;
+
+     return DFB_OK;
 }
 
 static DFBResult
@@ -508,7 +540,7 @@ IDirectFBImageProvider_JPEG_GetSurfaceDescription( IDirectFBImageProvider *thiz,
      struct jpeg_decompress_struct cinfo;
      struct my_error_mgr jerr;
 
-     INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
 
      cinfo.err = jpeg_std_error(&jerr.pub);
      jerr.pub.error_exit = jpeglib_panic;
@@ -537,7 +569,7 @@ static DFBResult
 IDirectFBImageProvider_JPEG_GetImageDescription( IDirectFBImageProvider *thiz,
                                                  DFBImageDescription    *dsc )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
 
      if (!dsc)
           return DFB_INVARG;
