@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,6 +25,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +35,6 @@
 #include <string.h>
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <display/idirectfbsurface.h>
 
@@ -46,8 +48,13 @@
 #include <core/surfaces.h>
 
 #include <misc/gfx_util.h>
-#include <misc/mem.h>
-#include <misc/memcpy.h>
+
+#include <direct/interface.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
 #include <misc/util.h>
 
 #include "config.h"
@@ -59,9 +66,9 @@ static DFBResult
 Construct( IDirectFBImageProvider *thiz,
            IDirectFBDataBuffer    *buffer );
 
-#include <interface_implementation.h>
+#include <direct/interface_implementation.h>
 
-DFB_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, PNG )
+DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, PNG )
 
 
 enum {
@@ -161,14 +168,14 @@ Construct( IDirectFBImageProvider *thiz,
 {
      DFBResult ret = DFB_FAILURE;
 
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_PNG)
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_PNG)
 
      data->ref    = 1;
      data->buffer = buffer;
 
      /* Increase the data buffer reference counter. */
      buffer->AddRef( buffer );
-     
+
      /* Create the PNG read handle. */
      data->png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING,
                                              NULL, NULL, NULL );
@@ -186,19 +193,18 @@ Construct( IDirectFBImageProvider *thiz,
                                   png_row_callback,
                                   png_end_callback );
 
-     
+
      /* Read until info callback is called. */
-     ret = push_data_until_stage( data, STAGE_INFO, 4 );
+     ret = push_data_until_stage( data, STAGE_INFO, 64 );
      if (ret)
           goto error;
 
-     thiz->AddRef = IDirectFBImageProvider_PNG_AddRef;
-     thiz->Release = IDirectFBImageProvider_PNG_Release;
-     thiz->RenderTo = IDirectFBImageProvider_PNG_RenderTo;
-     thiz->SetRenderCallback = IDirectFBImageProvider_PNG_SetRenderCallback;
-     thiz->GetImageDescription = IDirectFBImageProvider_PNG_GetImageDescription;
-     thiz->GetSurfaceDescription =
-                              IDirectFBImageProvider_PNG_GetSurfaceDescription;
+     thiz->AddRef                = IDirectFBImageProvider_PNG_AddRef;
+     thiz->Release               = IDirectFBImageProvider_PNG_Release;
+     thiz->RenderTo              = IDirectFBImageProvider_PNG_RenderTo;
+     thiz->SetRenderCallback     = IDirectFBImageProvider_PNG_SetRenderCallback;
+     thiz->GetImageDescription   = IDirectFBImageProvider_PNG_GetImageDescription;
+     thiz->GetSurfaceDescription = IDirectFBImageProvider_PNG_GetSurfaceDescription;
 
      return DFB_OK;
 
@@ -207,11 +213,11 @@ error:
           png_destroy_read_struct( &data->png_ptr, &data->info_ptr, NULL );
 
      buffer->Release( buffer );
-     
+
      if (data->image)
-          DFBFREE( data->image );
-     
-     DFB_DEALLOCATE_INTERFACE(thiz);
+          D_FREE( data->image );
+
+     DIRECT_DEALLOCATE_INTERFACE(thiz);
 
      return ret;
 }
@@ -223,21 +229,21 @@ IDirectFBImageProvider_PNG_Destruct( IDirectFBImageProvider *thiz )
                               (IDirectFBImageProvider_PNG_data*)thiz->priv;
 
      png_destroy_read_struct( &data->png_ptr, &data->info_ptr, NULL );
-     
+
      /* Decrease the data buffer reference counter. */
      data->buffer->Release( data->buffer );
 
      /* Deallocate image data. */
      if (data->image)
-          DFBFREE( data->image );
-     
-     DFB_DEALLOCATE_INTERFACE( thiz );
+          D_FREE( data->image );
+
+     DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 static DFBResult
 IDirectFBImageProvider_PNG_AddRef( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
 
      data->ref++;
 
@@ -247,7 +253,7 @@ IDirectFBImageProvider_PNG_AddRef( IDirectFBImageProvider *thiz )
 static DFBResult
 IDirectFBImageProvider_PNG_Release( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
 
      if (--data->ref == 0) {
           IDirectFBImageProvider_PNG_Destruct( thiz );
@@ -266,7 +272,7 @@ IDirectFBImageProvider_PNG_RenderTo( IDirectFBImageProvider *thiz,
      CoreSurface           *dst_surface;
      DFBRectangle           rect = { 0, 0, 0, 0 };
 
-     INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
 
      dst_data = (IDirectFBSurface_data*) destination->priv;
      if (!dst_data)
@@ -281,7 +287,7 @@ IDirectFBImageProvider_PNG_RenderTo( IDirectFBImageProvider *thiz,
           return ret;
 
      /* Read until image is completely decoded. */
-     ret = push_data_until_stage( data, STAGE_END, 4096 );
+     ret = push_data_until_stage( data, STAGE_END, 16384 );
      if (ret)
           return ret;
 
@@ -296,7 +302,7 @@ IDirectFBImageProvider_PNG_RenderTo( IDirectFBImageProvider *thiz,
 
           dfb_scale_linear_32( data->image, data->width, data->height,
                                dst, pitch, &rect, dst_surface );
-          
+
           destination->Unlock( destination );
      }
 
@@ -308,12 +314,12 @@ IDirectFBImageProvider_PNG_SetRenderCallback( IDirectFBImageProvider *thiz,
                                               DIRenderCallback        callback,
                                               void                   *context )
 {
-     INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
 
      data->render_callback         = callback;
      data->render_callback_context = context;
 
-     return DFB_UNIMPLEMENTED;
+     return DFB_OK;
 }
 
 /* Loading routines */
@@ -322,8 +328,8 @@ static DFBResult
 IDirectFBImageProvider_PNG_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                   DFBSurfaceDescription *dsc )
 {
-     INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
-          
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
+
      dsc->flags  = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
      dsc->width  = data->width;
      dsc->height = data->height;
@@ -332,7 +338,7 @@ IDirectFBImageProvider_PNG_GetSurfaceDescription( IDirectFBImageProvider *thiz,
           dsc->pixelformat = DSPF_ARGB;
      else
           dsc->pixelformat = dfb_primary_layer_pixelformat();
-     
+
      return DFB_OK;
 }
 
@@ -340,16 +346,16 @@ static DFBResult
 IDirectFBImageProvider_PNG_GetImageDescription( IDirectFBImageProvider *thiz,
                                                 DFBImageDescription    *dsc )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_PNG)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_PNG)
 
      if (!dsc)
           return DFB_INVARG;
 
      dsc->caps = DICAPS_NONE;
-     
+
      if (data->color_type & PNG_COLOR_MASK_ALPHA)
           dsc->caps |= DICAPS_ALPHACHANNEL;
-     
+
      if (data->color_keyed) {
           dsc->caps |= DICAPS_COLORKEY;
 
@@ -380,12 +386,12 @@ static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
      if (n_colors < 1)
           return color;
 
-     DFB_ASSERT( n_colors <= MAXCOLORMAPSIZE );
+     D_ASSERT( n_colors <= MAXCOLORMAPSIZE );
 
      for (i = 0; i < 3; i++) {
-          dfb_memcpy( csort, cmap[i], n_colors );
+          direct_memcpy( csort, cmap[i], n_colors );
           qsort( csort, n_colors, 1, SortColors );
-          
+
           for (j = 1, index = 0, d = 0; j < n_colors; j++) {
                if (csort[j] - csort[j-1] > d) {
                     d = csort[j] - csort[j-1];
@@ -399,7 +405,7 @@ static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
           if (0xFF - (csort[n_colors - 1]) > d) {
                index = n_colors + 1;
           }
-          
+
           if (index < n_colors)
                csort[0] = csort[index] - (d/2);
           else if (index == n_colors)
@@ -442,7 +448,7 @@ png_info_callback   (png_structp png_read_ptr,
 
      if (png_get_valid( data->png_ptr, data->info_ptr, PNG_INFO_tRNS )) {
           data->color_keyed = true;
-          
+
           /* generate color key based on palette... */
           if (data->color_type == PNG_COLOR_TYPE_PALETTE) {
                int        i;
@@ -464,7 +470,7 @@ png_info_callback   (png_structp png_read_ptr,
                for (i=0; i<data->info_ptr->num_trans; i++) {
                     if (!trans[i]) {
                          //trans[i] = 0;
-                         
+
                          palette[i].red   = (key & 0xff0000) >> 16;
                          palette[i].green = (key & 0x00ff00) >>  8;
                          palette[i].blue  = (key & 0x0000ff);
@@ -477,8 +483,8 @@ png_info_callback   (png_structp png_read_ptr,
                /* ...or based on trans rgb value */
                png_color_16p trans = &data->info_ptr->trans_values;
 
-               CAUTION("color key from non-palette source is untested");
-               
+               D_WARN("color key from non-palette source is untested");
+
                data->color_key = (((trans->red & 0xff00) << 8) |
                                   ((trans->green & 0xff00)) |
                                   ((trans->blue & 0xff00) >> 8));
@@ -499,7 +505,7 @@ png_info_callback   (png_structp png_read_ptr,
 
      png_set_bgr( data->png_ptr );
 #endif
-     
+
      png_set_interlace_handling( data->png_ptr );
 
      /* Update the info to reflect our transformations */
@@ -527,12 +533,15 @@ png_row_callback   (png_structp png_read_ptr,
 
      /* check image data pointer */
      if (!data->image) {
-          int size = data->width * data->height * 4;
+          // FIXME: allocates four additional bytes because the scaling functions
+          //        in src/misc/gfx_util.c have an off-by-one bug which causes
+          //        segfaults on darwin/osx (not on linux)                
+          int size = data->width * data->height * 4 + 4 ;
 
           /* allocate image data */
-          data->image = DFBMALLOC( size );
+          data->image = D_MALLOC( size );
           if (!data->image) {
-               ERRORMSG("DirectFB/ImageProvider_PNG: Could not "
+               D_ERROR("DirectFB/ImageProvider_PNG: Could not "
                         "allocate %d bytes of system memory!\n", size);
 
                /* set error stage */
@@ -548,6 +557,12 @@ png_row_callback   (png_structp png_read_ptr,
 
      /* increase row counter, FIXME: interlaced? */
      data->rows++;
+
+     if (data->render_callback) {
+          DFBRectangle rect = { 0, row_num, data->width, 1 };
+
+          data->render_callback( &rect, data->render_callback_context );
+     }
 }
 
 /* Called after reading the entire image */
@@ -562,7 +577,7 @@ png_end_callback   (png_structp png_read_ptr,
      /* error stage? */
      if (data->stage < 0)
           return;
-     
+
      /* set end stage */
      data->stage = STAGE_END;
 }
@@ -583,20 +598,28 @@ push_data_until_stage (IDirectFBImageProvider_PNG_data *data,
           if (data->stage < 0)
                return DFB_FAILURE;
 
-          if (buffer->WaitForData( buffer, 1 ) == DFB_BUFFEREMPTY)
-               return DFB_FAILURE;
-          
           while (buffer->HasData( buffer ) == DFB_OK) {
+               D_DEBUG( "ImageProvider/PNG: Retrieving data (up to %d bytes)...\n", buffer_size );
+
                ret = buffer->GetData( buffer, buffer_size, buf, &len );
                if (ret)
                     return ret;
 
+               D_DEBUG( "ImageProvider/PNG: Got %d bytes...\n", len );
+
                png_process_data( data->png_ptr, data->info_ptr, buf, len );
 
+               D_DEBUG( "ImageProvider/PNG: ...processed %d bytes.\n", len );
+
                /* are we there yet? */
                if (data->stage < 0 || data->stage >= stage)
-                    break;
+                    return DFB_OK;
           }
+
+          D_DEBUG( "ImageProvider/PNG: Waiting for data...\n" );
+
+          if (buffer->WaitForData( buffer, 1 ) == DFB_BUFFEREMPTY)
+               return DFB_FAILURE;
      }
 
      return DFB_OK;
