@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -31,13 +32,12 @@
 
 #include <directfb.h>
 
-#include <core/fusion/list.h>
-                                   
+#include <direct/list.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/core_parts.h>
 #include <core/layers.h>
-#include <core/modules.h>
 #include <core/palette.h>
 #include <core/surfaces.h>
 #include <core/system.h>
@@ -45,7 +45,10 @@
 #include <gfx/convert.h>
 
 #include <misc/conf.h>
-#include <misc/mem.h>
+
+#include <direct/mem.h>
+#include <direct/messages.h>
+#include <direct/modules.h>
 
 DEFINE_MODULE_DIRECTORY( dfb_core_systems, "systems",
                          DFB_CORE_SYSTEM_ABI_VERSION );
@@ -58,7 +61,7 @@ DFB_CORE_PART( system, 0, sizeof(CoreSystemField) )
 
 static CoreSystemField       *system_field  = NULL;
 
-static ModuleEntry           *system_module = NULL;
+static DirectModuleEntry     *system_module = NULL;
 static const CoreSystemFuncs *system_funcs  = NULL;
 static CoreSystemInfo         system_info;
 static void                  *system_data   = NULL;
@@ -67,15 +70,15 @@ static void                  *system_data   = NULL;
 DFBResult
 dfb_system_lookup()
 {
-     FusionLink *l;
+     DirectLink *l;
 
-     dfb_modules_explore_directory( &dfb_core_systems );
-     
-     fusion_list_foreach( l, dfb_core_systems.entries ) {
-          ModuleEntry           *module = (ModuleEntry*) l;
+     direct_modules_explore_directory( &dfb_core_systems );
+
+     direct_list_foreach( l, dfb_core_systems.entries ) {
+          DirectModuleEntry     *module = (DirectModuleEntry*) l;
           const CoreSystemFuncs *funcs;
 
-          funcs = dfb_module_ref( module );
+          funcs = direct_module_ref( module );
           if (!funcs)
                continue;
 
@@ -83,19 +86,19 @@ dfb_system_lookup()
               !strcasecmp( dfb_config->system, module->name )))
           {
                if (system_module)
-                    dfb_module_unref( system_module );
+                    direct_module_unref( system_module );
 
                system_module = module;
                system_funcs  = funcs;
-               
+
                funcs->GetSystemInfo( &system_info );
           }
           else
-               dfb_module_unref( module );
+               direct_module_unref( module );
      }
 
      if (!system_module) {
-          ERRORMSG("DirectFB/core/system: No system found!\n");
+          D_ERROR("DirectFB/core/system: No system found!\n");
 
           return DFB_NOIMPL;
      }
@@ -104,30 +107,30 @@ dfb_system_lookup()
 }
 
 static DFBResult
-dfb_system_initialize( void *data_local, void *data_shared )
+dfb_system_initialize( CoreDFB *core, void *data_local, void *data_shared )
 {
-     DFB_ASSERT( system_funcs != NULL );
-     DFB_ASSERT( system_field == NULL );
+     D_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_field == NULL );
 
      system_field = data_shared;
 
      system_field->system_info = system_info;
 
-     return system_funcs->Initialize( &system_data );
+     return system_funcs->Initialize( core, &system_data );
 }
 
 static DFBResult
-dfb_system_join( void *data_local, void *data_shared )
+dfb_system_join( CoreDFB *core, void *data_local, void *data_shared )
 {
-     DFB_ASSERT( system_funcs != NULL );
-     DFB_ASSERT( system_field == NULL );
+     D_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_field == NULL );
 
      system_field = data_shared;
-     
+
      if (system_field->system_info.type != system_info.type ||
          strcmp( system_field->system_info.name, system_info.name ))
      {
-          ERRORMSG( "DirectFB/core/system: "
+          D_ERROR( "DirectFB/core/system: "
                     "running system '%s' doesn't match system '%s'!\n",
                     system_field->system_info.name, system_info.name );
 
@@ -135,11 +138,11 @@ dfb_system_join( void *data_local, void *data_shared )
 
           return DFB_UNSUPPORTED;
      }
-     
+
      if (system_field->system_info.version.major != system_info.version.major ||
          system_field->system_info.version.minor != system_info.version.minor)
      {
-          ERRORMSG( "DirectFB/core/system: running system version '%d.%d' "
+          D_ERROR( "DirectFB/core/system: running system version '%d.%d' "
                     "doesn't match version '%d.%d'!\n",
                     system_field->system_info.version.major,
                     system_field->system_info.version.minor,
@@ -150,21 +153,21 @@ dfb_system_join( void *data_local, void *data_shared )
 
           return DFB_UNSUPPORTED;
      }
-     
-     return system_funcs->Join( &system_data );
+
+     return system_funcs->Join( core, &system_data );
 }
 
 static DFBResult
-dfb_system_shutdown( bool emergency )
+dfb_system_shutdown( CoreDFB *core, bool emergency )
 {
      DFBResult ret;
-     
-     DFB_ASSERT( system_field != NULL );
-     DFB_ASSERT( system_module != NULL );
-     
+
+     D_ASSERT( system_field != NULL );
+     D_ASSERT( system_module != NULL );
+
      ret = system_funcs->Shutdown( emergency );
 
-     dfb_module_unref( system_module );
+     direct_module_unref( system_module );
 
      system_module = NULL;
      system_funcs  = NULL;
@@ -175,16 +178,16 @@ dfb_system_shutdown( bool emergency )
 }
 
 static DFBResult
-dfb_system_leave( bool emergency )
+dfb_system_leave( CoreDFB *core, bool emergency )
 {
      DFBResult ret;
-     
-     DFB_ASSERT( system_field != NULL );
-     DFB_ASSERT( system_module != NULL );
-     
+
+     D_ASSERT( system_field != NULL );
+     D_ASSERT( system_module != NULL );
+
      ret = system_funcs->Leave( emergency );
 
-     dfb_module_unref( system_module );
+     direct_module_unref( system_module );
 
      system_module = NULL;
      system_funcs  = NULL;
@@ -195,19 +198,19 @@ dfb_system_leave( bool emergency )
 }
 
 static DFBResult
-dfb_system_suspend()
+dfb_system_suspend( CoreDFB *core )
 {
-     DFB_ASSERT( system_funcs != NULL );
-     DFB_ASSERT( system_field != NULL );
+     D_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_field != NULL );
 
      return system_funcs->Suspend();
 }
 
 static DFBResult
-dfb_system_resume()
+dfb_system_resume( CoreDFB *core )
 {
-     DFB_ASSERT( system_funcs != NULL );
-     DFB_ASSERT( system_field != NULL );
+     D_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_field != NULL );
 
      return system_funcs->Resume();
 }
@@ -228,7 +231,7 @@ volatile void *
 dfb_system_map_mmio( unsigned int    offset,
                      int             length )
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->MapMMIO( offset, length );
 }
@@ -237,7 +240,7 @@ void
 dfb_system_unmap_mmio( volatile void  *addr,
                        int             length )
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      system_funcs->UnmapMMIO( addr, length );
 }
@@ -245,7 +248,7 @@ dfb_system_unmap_mmio( volatile void  *addr,
 int
 dfb_system_get_accelerator()
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->GetAccelerator();
 }
@@ -253,7 +256,7 @@ dfb_system_get_accelerator()
 VideoMode *
 dfb_system_modes()
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->GetModes();
 }
@@ -261,7 +264,7 @@ dfb_system_modes()
 VideoMode *
 dfb_system_current_mode()
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->GetCurrentMode();
 }
@@ -269,16 +272,16 @@ dfb_system_current_mode()
 DFBResult
 dfb_system_thread_init()
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->ThreadInit();
 }
 
 bool
-dfb_system_input_filter( InputDevice   *device,
-                         DFBInputEvent *event )
+dfb_system_input_filter( CoreInputDevice *device,
+                         DFBInputEvent   *event )
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->InputFilter( device, event );
 }
@@ -286,7 +289,7 @@ dfb_system_input_filter( InputDevice   *device,
 unsigned long
 dfb_system_video_memory_physical( unsigned int offset )
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->VideoMemoryPhysical( offset );
 }
@@ -294,7 +297,7 @@ dfb_system_video_memory_physical( unsigned int offset )
 void *
 dfb_system_video_memory_virtual( unsigned int offset )
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->VideoMemoryVirtual( offset );
 }
@@ -302,7 +305,7 @@ dfb_system_video_memory_virtual( unsigned int offset )
 unsigned int
 dfb_system_videoram_length()
 {
-     DFB_ASSERT( system_funcs != NULL );
+     D_ASSERT( system_funcs != NULL );
 
      return system_funcs->VideoRamLength();
 }
