@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2002-2004  convergence GmbH.
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,7 +25,14 @@
    Boston, MA 02111-1307, USA.
 */
 
-static void Cacc_add_to_Dacc_MMX( GenefxState *gfxs )
+#ifdef __GNUC__
+# define __aligned( n )  __attribute__ ((aligned((n))))
+#else
+# define __aligned( n )
+#endif
+
+
+static void SCacc_add_to_Dacc_MMX( GenefxState *gfxs )
 {
      __asm__ __volatile__ (
                "    movq     %2, %%mm0\n"
@@ -38,7 +46,7 @@ static void Cacc_add_to_Dacc_MMX( GenefxState *gfxs )
                "    jnz      1b\n"
                "    emms"
                : /* no outputs */
-               : "D" (gfxs->Dacc), "c" (gfxs->length), "m" (gfxs->Cacc)
+               : "D" (gfxs->Dacc), "c" (gfxs->length), "m" (gfxs->SCacc)
                : "%st", "memory");
 }
 
@@ -156,6 +164,7 @@ static void Sacc_to_Aop_rgb32_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+__attribute__((no_instrument_function))
 static void Sop_argb_Sto_Dacc_MMX( GenefxState *gfxs )
 {
      static const long zeros[]  = { 0, 0 };
@@ -176,10 +185,10 @@ static void Sop_argb_Sto_Dacc_MMX( GenefxState *gfxs )
                "addl     %4, %0\n\t"
                "testl    $0xFFFF0000, %0\n\t"
                "jz       2b\n\t"
-               "movl     %0, %%ebx\n\t"
-               "andl     $0xFFFF0000, %%ebx\n\t"
-               "shrl     $14, %%ebx\n\t"
-               "add      %%ebx, %3\n\t"
+               "movl     %0, %%edx\n\t"
+               "andl     $0xFFFF0000, %%edx\n\t"
+               "shrl     $14, %%edx\n\t"
+               "add      %%edx, %3\n\t"
                "andl     $0xFFFF, %0\n\t"
                "jmp      1b\n"
                "3:\n\t"
@@ -187,7 +196,7 @@ static void Sop_argb_Sto_Dacc_MMX( GenefxState *gfxs )
                : "=r" (i)
                : "D" (gfxs->Dacc), "c" (gfxs->length), "S" (gfxs->Sop),
                  "a" (gfxs->SperD), "m" (*zeros), "0" (i)
-               : "%st", "memory");
+               : "%edx", "%st", "memory");
 }
 
 static void Sop_argb_to_Dacc_MMX( GenefxState *gfxs )
@@ -212,6 +221,124 @@ static void Sop_argb_to_Dacc_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+static void Sop_yuy2_to_Dacc_MMX( GenefxState *gfxs )
+{
+     static __u16 __aligned(8) mask[]  = {  0xFFFF,       0,  0xFFFF,       0 };
+     static __u16 __aligned(8) sub[]   = {      16,     128,      16,     128 };   
+     static __s16 __aligned(8) mul0[]  = {  0x253F,  0x3312,  0x253F,  0x4093 };
+     static __s16 __aligned(8) mul1[]  = { -0x0C83, -0x1A04, -0x0C83, -0x1A04 };
+     static __u16 __aligned(8) alpha[] = {  0xFF00,  0xFF00,  0xFF00,  0xFF00 };
+     
+     __asm__ __volatile__ (
+               "shrl      $1, %1\n\t"       // gfx->length /= 2
+               "jz        2f\n\t"           // return if 0
+               "pxor      %%mm7, %%mm7\n\t"
+               ".align 16\n"
+               "1:\n\t"
+               "movd       (%2), %%mm0\n\t" // 00 00 00 00 cr y1 cb y0
+               "punpcklbw %%mm7, %%mm0\n\t" // 00 cr 00 y1 00 cb 00 y0
+               "psubw        %3, %%mm0\n\t" // luma -= 16, chroma -= 128 
+               "psllw        $3, %%mm0\n\t" // precision
+               "movq      %%mm0, %%mm1\n\t" // save
+                
+               "pmulhw       %5, %%mm0\n\t" // 00 vr 00 y1 00 ub 00 y0
+               "movq      %%mm0, %%mm2\n\t"
+               "pand         %4, %%mm2\n\t" // 00 00 00 y1 00 00 00 y0
+               "movq      %%mm2, %%mm3\n\t"
+               "pslld       $16, %%mm3\n\t" // 00 y1 00 00 00 y0 00 00
+               "por       %%mm3, %%mm2\n\t" // 00 y1 00 y1 00 y0 00 y0
+               "psrad       $16, %%mm0\n\t" // 00 00 00 vr 00 00 00 ub
+               "packssdw  %%mm0, %%mm0\n\t" // 00 vr 00 ub 00 vr 00 ub
+               "paddw     %%mm2, %%mm0\n\t" // 00 r1 00 b1 00 r0 00 b0
+               "packuswb  %%mm0, %%mm0\n\t" // r1 b1 r0 b0 r1 b1 r0 b0
+               
+               "psrad       $16, %%mm1\n\t" // 00 00 00 cr 00 00 00 cb
+               "packssdw  %%mm1, %%mm1\n\t" // 00 cr 00 cb 00 cr 00 cb
+               "pmulhw       %6, %%mm1\n\t" // 00 vg 00 ug 00 vg 00 ug
+               "movq      %%mm1, %%mm3\n\t"
+               "psrld       $16, %%mm3\n\t" // 00 00 00 vg 00 00 00 vg
+               "paddw     %%mm3, %%mm1\n\t"
+               "paddw     %%mm1, %%mm2\n\t" // 00 XX 00 g1 00 XX 00 g0
+               "packuswb  %%mm2, %%mm2\n\t" // XX g1 XX g0 XX g1 XX g0
+               "por          %7, %%mm2\n\t" // FF g1 FF g0 FF g1 FF g0
+               
+               "punpcklbw %%mm2, %%mm0\n\t" // FF r1 g1 b1 FF r0 g0 b0
+               "movq      %%mm0, %%mm1\n\t"
+               "punpcklbw %%mm7, %%mm0\n\t" // 00 FF 00 r0 00 g0 00 b0
+               "punpckhbw %%mm7, %%mm1\n\t" // 00 FF 00 r1 00 g1 00 b1
+               "movq      %%mm0,  (%0)\n\t"
+               "movq      %%mm1, 8(%0)\n\t"
+               "addl         $4,    %2\n\t"
+               "addl        $16,    %0\n\t"
+               "decl         %1\n\t"
+               "jnz          1b\n\t"
+               "emms\n\t"
+               "2:"
+               : /* no outputs */
+               : "D" (gfxs->Dacc), "c" (gfxs->length), "S" (gfxs->Sop),
+                 "m" (*sub), "m" (*mask), "m" (*mul0), "m" (*mul1), "m" (*alpha)
+               : "memory" );
+}
+
+static void Sop_uyvy_to_Dacc_MMX( GenefxState *gfxs )
+{
+     static __u16 __aligned(8) sub[]   = {     128,      16,     128,      16 };   
+     static __s16 __aligned(8) mul0[]  = {  0x3312,  0x253F,  0x4093,  0x253F };
+     static __s16 __aligned(8) mul1[]  = { -0x0C83, -0x1A04, -0x0C83, -0x1A04 };
+     static __u16 __aligned(8) alpha[] = {  0xFF00,  0xFF00,  0xFF00,  0xFF00 };
+      
+     __asm__ __volatile__ (
+               "shrl      $1, %1\n\t"       // gfx->length /= 2
+               "jz        2f\n\t"           // return if 0
+               "pxor      %%mm7, %%mm7\n\t"
+               ".align 16\n"
+               "1:\n\t"
+               "movd       (%2), %%mm0\n\t" // 00 00 00 00 y1 cr y0 cb
+               "punpcklbw %%mm7, %%mm0\n\t" // 00 y1 00 cr 00 y0 00 cb
+               "psubw        %3, %%mm0\n\t" // luma -= 16, chroma -= 128 
+               "psllw        $3, %%mm0\n\t" // precision
+               "movq      %%mm0, %%mm1\n\t" // save
+                
+               "pmulhw       %4, %%mm0\n\t" // 00 y1 00 vr 00 y0 00 ub
+               "movq      %%mm0, %%mm2\n\t"
+               "psrad       $16, %%mm2\n\t" // 00 00 00 y1 00 00 00 y0
+               "packssdw  %%mm2, %%mm2\n\t" // 00 y1 00 y0 00 y1 00 y0
+               "punpcklwd %%mm2, %%mm2\n\t" // 00 y1 00 y1 00 y0 00 y0
+               "pslld       $16, %%mm0\n\t" // 00 vr 00 00 00 ub 00 00
+               "psrad       $16, %%mm0\n\t" // 00 00 00 vr 00 00 00 ub (sign extend)
+               "packssdw  %%mm0, %%mm0\n\t" // 00 vr 00 ub 00 vr 00 ub
+               "paddw     %%mm2, %%mm0\n\t" // 00 r1 00 b1 00 r0 00 b0
+               "packuswb  %%mm0, %%mm0\n\t" // r1 b1 r0 b0 r1 b1 r0 b0
+               
+               "pslld       $16, %%mm1\n\t" // 00 cr 00 00 00 cb 00 00
+               "psrad       $16, %%mm1\n\t" // 00 00 00 cr 00 00 00 cb (sign extend)
+               "packssdw  %%mm1, %%mm1\n\t" // 00 cr 00 cb 00 cr 00 cb
+               "pmulhw       %5, %%mm1\n\t" // 00 vg 00 ug 00 vg 00 ug
+               "movq      %%mm1, %%mm3\n\t"
+               "psrld       $16, %%mm3\n\t" // 00 00 00 vg 00 00 00 vg
+               "paddw     %%mm3, %%mm1\n\t"
+               "paddw     %%mm1, %%mm2\n\t" // 00 XX 00 g1 00 XX 00 g0
+               "packuswb  %%mm2, %%mm2\n\t" // XX g1 XX g0 XX g1 XX g0
+               "por          %6, %%mm2\n\t" // FF g1 FF g0 FF g1 FF g0
+               
+               "punpcklbw %%mm2, %%mm0\n\t" // FF r1 g1 b1 FF r0 g0 b0
+               "movq      %%mm0, %%mm1\n\t"
+               "punpcklbw %%mm7, %%mm0\n\t" // 00 FF 00 r0 00 g0 00 b0
+               "punpckhbw %%mm7, %%mm1\n\t" // 00 FF 00 r1 00 g1 00 b1
+               "movq      %%mm0,  (%0)\n\t"
+               "movq      %%mm1, 8(%0)\n\t"
+               "addl         $4,    %2\n\t"
+               "addl        $16,    %0\n\t"
+               "decl         %1\n\t"
+               "jnz          1b\n\t"
+               "emms\n\t"
+               "2:"
+               : /* no outputs */
+               : "D" (gfxs->Dacc), "c" (gfxs->length), "S" (gfxs->Sop),
+                 "m" (*sub), "m" (*mul0), "m" (*mul1), "m" (*alpha)
+               : "memory" );
+}
+
 static void Sop_rgb16_to_Dacc_MMX( GenefxState *gfxs )
 {
      static const long mask[]  = { 0x07E0001F, 0x0000F800 };
