@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -30,14 +31,9 @@
 #include <core/surfaces.h>
 
 void dfb_gfx_copy( CoreSurface *source, CoreSurface *destination, DFBRectangle *rect );
-void dfb_back_to_front_copy( CoreSurface *surface, DFBRectangle *rect );
+void dfb_back_to_front_copy( CoreSurface *surface, const DFBRegion *region );
+void dfb_clear_depth( CoreSurface *surface, const DFBRegion *region );
 
 void dfb_sort_triangle( DFBTriangle *tri );
 
-static inline bool dfb_colors_equal( const DFBColor *c1, const DFBColor *c2 )
-{
-     return ((c1->a == c2->a) && (c1->r == c2->r) &&
-             (c1->g == c2->g) && (c1->b == c2->b));
-}
-
 #endif
