@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2003  convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,11 +30,20 @@
 
 #include <directfb.h>
 
+#include <core/coretypes.h>
+
 /*
  * IDirectFB constructor/destructor
  */
-DFBResult IDirectFB_Construct( IDirectFB *thiz );
+DFBResult IDirectFB_Construct  ( IDirectFB  *thiz,
+                                 CoreDFB    *core );
 
-DFBResult IDirectFB_SetAppFocus( IDirectFB *thiz, DFBBoolean focused );
+void      IDirectFB_Destruct   ( IDirectFB  *thiz );
+
+DFBResult IDirectFB_SetAppFocus( IDirectFB  *thiz,
+                                 DFBBoolean  focused );
+
+
+extern IDirectFB *idirectfb_singleton;
 
 #endif
