@@ -1,7 +1,7 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
    (c) Copyright 2002       convergence GmbH.
-   
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
@@ -37,23 +37,24 @@
 
 #include <pthread.h>
 
-#include <core/fusion/reactor.h>
-#include <core/fusion/list.h>
+#include <fusion/reactor.h>
+#include <direct/list.h>
+#include <fusion/lock.h>
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
-#include <misc/util.h>
-#include <misc/mem.h>
-#include <misc/memcpy.h>
+#include <direct/interface.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
+#include <direct/util.h>
 
 #include <media/idirectfbdatabuffer.h>
 
 typedef struct {
-     FusionLink    link;
+     DirectLink    link;
 
      void         *data;   /* actual data hold */
      unsigned int  length; /* length of chunk */
@@ -73,9 +74,7 @@ destroy_chunk( DataChunk *chunk );
 typedef struct {
      IDirectFBDataBuffer_data  base;
 
-     FusionLink               *chunks;          /* data chunks */
-     FusionLink               *last;            /* last chunk is the
-                                                   first to read */
+     DirectLink               *chunks;          /* data chunks */
 
      unsigned int              length;          /* total length of all chunks */
 
@@ -102,17 +101,17 @@ IDirectFBDataBuffer_Streamed_Destruct( IDirectFBDataBuffer *thiz )
 {
      IDirectFBDataBuffer_Streamed_data *data =
           (IDirectFBDataBuffer_Streamed_data*) thiz->priv;
-     
+
      pthread_cond_destroy( &data->wait_condition );
      pthread_mutex_destroy( &data->chunks_mutex );
-     
+
      IDirectFBDataBuffer_Destruct( thiz );
 }
 
 static DFBResult
 IDirectFBDataBuffer_Streamed_Release( IDirectFBDataBuffer *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBDataBuffer)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer)
 
      if (--data->ref == 0)
           IDirectFBDataBuffer_Streamed_Destruct( thiz );
@@ -123,14 +122,14 @@ IDirectFBDataBuffer_Streamed_Release( IDirectFBDataBuffer *thiz )
 static DFBResult
 IDirectFBDataBuffer_Streamed_Flush( IDirectFBDataBuffer *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      pthread_mutex_lock( &data->chunks_mutex );
-     
+
      DestroyAllChunks( data );
-     
+
      pthread_mutex_unlock( &data->chunks_mutex );
-     
+
      return DFB_OK;
 }
 
@@ -152,7 +151,7 @@ static DFBResult
 IDirectFBDataBuffer_Streamed_GetLength( IDirectFBDataBuffer *thiz,
                                         unsigned int        *length )
 {
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      /* Check arguments. */
      if (!length)
@@ -160,7 +159,7 @@ IDirectFBDataBuffer_Streamed_GetLength( IDirectFBDataBuffer *thiz,
 
      /* Return total length of all chunks. */
      *length = data->length;
-     
+
      return DFB_OK;
 }
 
@@ -168,15 +167,15 @@ static DFBResult
 IDirectFBDataBuffer_Streamed_WaitForData( IDirectFBDataBuffer *thiz,
                                           unsigned int         length )
 {
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
-     
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+
      pthread_mutex_lock( &data->chunks_mutex );
 
      while (data->length < length)
           pthread_cond_wait( &data->wait_condition, &data->chunks_mutex );
 
      pthread_mutex_unlock( &data->chunks_mutex );
-     
+
      return DFB_OK;
 }
 
@@ -192,7 +191,7 @@ IDirectFBDataBuffer_Streamed_WaitForDataWithTimeout( IDirectFBDataBuffer *thiz,
      bool            locked       = false;
      long int        nano_seconds = milli_seconds * 1000000;
 
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      if (pthread_mutex_trylock( &data->chunks_mutex ) == 0) {
           if (data->length >= length) {
@@ -219,7 +218,10 @@ IDirectFBDataBuffer_Streamed_WaitForDataWithTimeout( IDirectFBDataBuffer *thiz,
           if (pthread_cond_timedwait( &data->wait_condition,
                                       &data->chunks_mutex,
                                       &timeout ) == ETIMEDOUT)
+          {
                ret = DFB_TIMEOUT;
+               break;
+          }
      }
 
      pthread_mutex_unlock( &data->chunks_mutex );
@@ -235,13 +237,13 @@ IDirectFBDataBuffer_Streamed_GetData( IDirectFBDataBuffer *thiz,
 {
      unsigned int len;
 
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      if (!data_buffer || !length)
           return DFB_INVARG;
 
      pthread_mutex_lock( &data->chunks_mutex );
-     
+
      if (!data->chunks) {
           pthread_mutex_unlock( &data->chunks_mutex );
           return DFB_BUFFEREMPTY;
@@ -259,9 +261,9 @@ IDirectFBDataBuffer_Streamed_GetData( IDirectFBDataBuffer *thiz,
      /* Return number of bytes read. */
      if (read_out)
           *read_out = len;
-     
+
      pthread_mutex_unlock( &data->chunks_mutex );
-     
+
      return DFB_OK;
 }
 
@@ -274,13 +276,13 @@ IDirectFBDataBuffer_Streamed_PeekData( IDirectFBDataBuffer *thiz,
 {
      unsigned int len;
 
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      if (!data_buffer || !length || offset < 0)
           return DFB_INVARG;
 
      pthread_mutex_lock( &data->chunks_mutex );
-     
+
      if (!data->chunks || (unsigned int) offset >= data->length) {
           pthread_mutex_unlock( &data->chunks_mutex );
           return DFB_BUFFEREMPTY;
@@ -295,21 +297,21 @@ IDirectFBDataBuffer_Streamed_PeekData( IDirectFBDataBuffer *thiz,
      /* Return number of bytes read. */
      if (read_out)
           *read_out = len;
-     
+
      pthread_mutex_unlock( &data->chunks_mutex );
-     
+
      return DFB_OK;
 }
 
 static DFBResult
 IDirectFBDataBuffer_Streamed_HasData( IDirectFBDataBuffer *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      /* If there's no chunk there's no data. */
      if (!data->chunks)
           return DFB_BUFFEREMPTY;
-     
+
      return DFB_OK;
 }
 
@@ -320,7 +322,7 @@ IDirectFBDataBuffer_Streamed_PutData( IDirectFBDataBuffer *thiz,
 {
      DataChunk *chunk;
 
-     INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Streamed)
 
      /* Check arguments. */
      if (!data_buffer || !length)
@@ -332,21 +334,17 @@ IDirectFBDataBuffer_Streamed_PutData( IDirectFBDataBuffer *thiz,
           return DFB_NOSYSTEMMEMORY;
 
      pthread_mutex_lock( &data->chunks_mutex );
-     
-     /* Prepend new chunk. */
-     fusion_list_prepend( &data->chunks, &chunk->link );
 
-     /* If no chunk has been there before it's the last one. */
-     if (!data->last)
-          data->last = data->chunks;
-     
+     /* Append new chunk. */
+     direct_list_append( &data->chunks, &chunk->link );
+
      /* Increase total length. */
      data->length += length;
 
      pthread_cond_broadcast( &data->wait_condition );
-     
+
      pthread_mutex_unlock( &data->chunks_mutex );
-     
+
      return DFB_OK;
 }
 
@@ -355,15 +353,15 @@ IDirectFBDataBuffer_Streamed_Construct( IDirectFBDataBuffer *thiz )
 {
      DFBResult ret;
 
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDataBuffer_Streamed)
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDataBuffer_Streamed)
 
      ret = IDirectFBDataBuffer_Construct( thiz, NULL );
      if (ret)
           return ret;
 
-     pthread_mutex_init( &data->chunks_mutex, NULL );
+     direct_util_recursive_pthread_mutex_init( &data->chunks_mutex );
      pthread_cond_init( &data->wait_condition, NULL );
-     
+
      thiz->Release                = IDirectFBDataBuffer_Streamed_Release;
      thiz->Flush                  = IDirectFBDataBuffer_Streamed_Flush;
      thiz->SeekTo                 = IDirectFBDataBuffer_Streamed_SeekTo;
@@ -384,26 +382,18 @@ IDirectFBDataBuffer_Streamed_Construct( IDirectFBDataBuffer *thiz )
 static void
 DestroyAllChunks( IDirectFBDataBuffer_Streamed_data *data )
 {
-     FusionLink *l;
+     DirectLink *l, *n;
 
-     DFB_ASSERT( data != NULL );
-
-     /* Fetch first link. */
-     l = data->chunks;
-
-     /* Loop through links. */
-     while (l) {
-          DataChunk *chunk = (DataChunk*) l;
-
-          /* Fetch next link. */
-          l = l->next;
+     D_ASSERT( data != NULL );
 
+     /* Loop through list. */
+     direct_list_foreach_safe (l, n, data->chunks) {
           /* Deallocate chunk. */
-          destroy_chunk( chunk );
+          destroy_chunk( (DataChunk*) l );
      }
 
-     /* Clear links. */
-     data->last = data->chunks = NULL;
+     /* Clear lists. */
+     data->chunks = NULL;
 }
 
 static void
@@ -413,20 +403,16 @@ ReadChunkData( IDirectFBDataBuffer_Streamed_data *data,
                unsigned int                       length,
                bool                               flush )
 {
-     FusionLink *l;
+     DirectLink *l, *n;
 
-     DFB_ASSERT( data != NULL );
-     DFB_ASSERT( buffer != NULL );
-
-     /* Fetch last link (the first to read). */
-     l = data->last;
+     D_ASSERT( data != NULL );
+     D_ASSERT( buffer != NULL );
 
      /* Loop through links. */
-     while (l && length) {
+     direct_list_foreach_safe (l, n, data->chunks) {
           unsigned int  len;
           unsigned int  off   = 0;
           DataChunk    *chunk = (DataChunk*) l;
-          FusionLink   *prev  = l->prev;
 
           /* Is there data to be skipped? */
           if (offset) {
@@ -443,7 +429,7 @@ ReadChunkData( IDirectFBDataBuffer_Streamed_data *data,
           /* Can we read from this chunk? */
           if (len) {
                /* Copy as many bytes as possible. */
-               dfb_memcpy( buffer, chunk->data + chunk->done + off, len );
+               direct_memcpy( buffer, chunk->data + chunk->done + off, len );
 
                /* Increase write pointer. */
                buffer += len;
@@ -459,25 +445,17 @@ ReadChunkData( IDirectFBDataBuffer_Streamed_data *data,
 
                /* Completely consumed? */
                if (chunk->done == chunk->length) {
-                    /* If this chunk is the last,
-                       the previous is the new last. */
-                    if (data->last == l)
-                         data->last = prev;
-
                     /* Remove the chunk from the list. */
-                    fusion_list_remove( &data->chunks, l );
+                    direct_list_remove( &data->chunks, l );
 
                     /* Deallocate chunk. */
                     destroy_chunk( chunk );
                }
           }
-          
-          /* Proceed with previous link. */
-          l = prev;
      }
 
-     DFB_ASSERT( length == 0 );
-     DFB_ASSERT( offset == 0 );
+     D_ASSERT( length == 0 );
+     D_ASSERT( offset == 0 );
 }
 
 /******************************************************************************/
@@ -487,23 +465,23 @@ create_chunk( const void *data, int length )
 {
      DataChunk *chunk;
 
-     DFB_ASSERT( data != NULL );
-     DFB_ASSERT( length > 0 );
+     D_ASSERT( data != NULL );
+     D_ASSERT( length > 0 );
 
      /* Allocate chunk information. */
-     chunk = DFBCALLOC( 1, sizeof(DataChunk) );
+     chunk = D_CALLOC( 1, sizeof(DataChunk) );
      if (!chunk)
           return NULL;
 
      /* Allocate chunk data. */
-     chunk->data = DFBMALLOC( length );
+     chunk->data = D_MALLOC( length );
      if (!chunk->data) {
-          DFBFREE( chunk );
+          D_FREE( chunk );
           return NULL;
      }
 
      /* Fill chunk data. */
-     dfb_memcpy( chunk->data, data, length );
+     direct_memcpy( chunk->data, data, length );
 
      /* Remember chunk length. */
      chunk->length = length;
@@ -514,13 +492,13 @@ create_chunk( const void *data, int length )
 static void
 destroy_chunk( DataChunk *chunk )
 {
-     DFB_ASSERT( chunk != NULL );
-     DFB_ASSERT( chunk->data != NULL );
+     D_ASSERT( chunk != NULL );
+     D_ASSERT( chunk->data != NULL );
 
      /* Deallocate chunk data. */
-     DFBFREE( chunk->data );
+     D_FREE( chunk->data );
 
      /* Deallocate chunk information. */
-     DFBFREE( chunk );
+     D_FREE( chunk );
 }
 
