@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -39,7 +40,6 @@
 #include <pthread.h>
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <idirectfb.h>
 
@@ -52,6 +52,8 @@
 #include <core/state.h>
 #include <core/surfaces.h>
 #include <core/windows.h>
+#include <core/windowstack.h>
+#include <core/windows_internal.h> /* FIXME */
 
 #include <display/idirectfbsurface.h>
 #include <display/idirectfbsurface_window.h>
@@ -59,12 +61,17 @@
 #include <input/idirectfbinputbuffer.h>
 
 #include <misc/util.h>
-#include <misc/mem.h>
+#include <direct/interface.h>
+#include <direct/mem.h>
 
 #include <gfx/convert.h>
 
 #include <windows/idirectfbwindow.h>
 
+
+D_DEBUG_DOMAIN( IDirectFB_Window, "IDirectFBWindow", "DirectFB Window Interface" );
+
+
 /*
  * adds an window event to the event queue
  */
@@ -76,11 +83,9 @@ static ReactionResult IDirectFBWindow_React( const void *msg_data,
 typedef struct {
      int                ref;
      CoreWindow        *window;
-     DisplayLayer      *layer;
+     CoreLayer         *layer;
 
      IDirectFBSurface  *surface;
-     
-     DFBWindowEvent    *position_size_event;
 
      struct {
           IDirectFBSurface  *shape;
@@ -102,42 +107,39 @@ IDirectFBWindow_Destruct( IDirectFBWindow *thiz )
 {
      IDirectFBWindow_data *data = (IDirectFBWindow_data*)thiz->priv;
 
-     DEBUGMSG("IDirectFBWindow_Destruct...\n");
-     
+     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_Destruct()\n" );
+
      if (!data->detached) {
-          DEBUGMSG("IDirectFBWindow_Destruct - detaching...\n");
-          
+          D_DEBUG_AT( IDirectFB_Window, "  -> detaching...\n" );
+
           dfb_window_detach( data->window, &data->reaction );
-          
-          DEBUGMSG("IDirectFBWindow_Destruct - detached.\n");
      }
-     
+
      if (!data->destroyed) {
-          DEBUGMSG("IDirectFBWindow_Destruct - unrefing...\n");
-          
+          D_DEBUG_AT( IDirectFB_Window, "  -> unrefing...\n" );
+
           dfb_window_unref( data->window );
-          
-          DEBUGMSG("IDirectFBWindow_Destruct - unref done.\n");
      }
 
+     D_DEBUG_AT( IDirectFB_Window, "  -> releasing surface...\n" );
+
      if (data->surface)
           data->surface->Release( data->surface );
 
+     D_DEBUG_AT( IDirectFB_Window, "  -> releasing cursor shape...\n" );
+
      if (data->cursor.shape)
           data->cursor.shape->Release( data->cursor.shape );
-     
-     if (data->position_size_event)
-          DFBFREE( data->position_size_event );
-     
-     DEBUGMSG("IDirectFBWindow_Destruct - done.\n");
 
-     DFB_DEALLOCATE_INTERFACE( thiz );
+     D_DEBUG_AT( IDirectFB_Window, "  -> done.\n" );
+
+     DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 static DFBResult
 IDirectFBWindow_AddRef( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      data->ref++;
 
@@ -147,7 +149,7 @@ IDirectFBWindow_AddRef( IDirectFBWindow *thiz )
 static DFBResult
 IDirectFBWindow_Release( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (--data->ref == 0)
           IDirectFBWindow_Destruct( thiz );
@@ -161,22 +163,18 @@ IDirectFBWindow_CreateEventBuffer( IDirectFBWindow       *thiz,
 {
      IDirectFBEventBuffer *b;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     DFB_ALLOCATE_INTERFACE( b, IDirectFBEventBuffer );
+     DIRECT_ALLOCATE_INTERFACE( b, IDirectFBEventBuffer );
 
      IDirectFBEventBuffer_Construct( b, NULL, NULL );
 
      IDirectFBEventBuffer_AttachWindow( b, data->window );
 
-     if (data->position_size_event) {
-          dfb_window_post_event( data->window, data->position_size_event );
-          DFBFREE( data->position_size_event );
-          data->position_size_event = NULL;
-     }
+     dfb_window_send_configuration( data->window );
 
      *buffer = b;
 
@@ -187,18 +185,14 @@ static DFBResult
 IDirectFBWindow_AttachEventBuffer( IDirectFBWindow       *thiz,
                                    IDirectFBEventBuffer  *buffer )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
      IDirectFBEventBuffer_AttachWindow( buffer, data->window );
 
-     if (data->position_size_event) {
-          dfb_window_post_event( data->window, data->position_size_event );
-          DFBFREE( data->position_size_event );
-          data->position_size_event = NULL;
-     }
+     dfb_window_send_configuration( data->window );
 
      return DFB_OK;
 }
@@ -207,7 +201,7 @@ static DFBResult
 IDirectFBWindow_EnableEvents( IDirectFBWindow       *thiz,
                               DFBWindowEventType     mask )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -215,16 +209,14 @@ IDirectFBWindow_EnableEvents( IDirectFBWindow       *thiz,
      if (mask & ~DWET_ALL)
           return DFB_INVARG;
 
-     data->window->events |= mask;
-
-     return DFB_OK;
+     return dfb_window_change_events( data->window, DWET_NONE, mask );
 }
 
 static DFBResult
 IDirectFBWindow_DisableEvents( IDirectFBWindow       *thiz,
                                DFBWindowEventType     mask )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -232,16 +224,14 @@ IDirectFBWindow_DisableEvents( IDirectFBWindow       *thiz,
      if (mask & ~DWET_ALL)
           return DFB_INVARG;
 
-     data->window->events &= ~mask;
-
-     return DFB_OK;
+     return dfb_window_change_events( data->window, mask, DWET_NONE );
 }
 
 static DFBResult
 IDirectFBWindow_GetID( IDirectFBWindow *thiz,
                        DFBWindowID     *id )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -259,7 +249,7 @@ IDirectFBWindow_GetPosition( IDirectFBWindow *thiz,
                              int             *x,
                              int             *y )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -268,10 +258,10 @@ IDirectFBWindow_GetPosition( IDirectFBWindow *thiz,
           return DFB_INVARG;
 
      if (x)
-          *x = data->window->x;
+          *x = data->window->config.bounds.x;
 
      if (y)
-          *y = data->window->y;
+          *y = data->window->config.bounds.y;
 
      return DFB_OK;
 }
@@ -281,7 +271,7 @@ IDirectFBWindow_GetSize( IDirectFBWindow *thiz,
                          int             *width,
                          int             *height )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -290,10 +280,10 @@ IDirectFBWindow_GetSize( IDirectFBWindow *thiz,
           return DFB_INVARG;
 
      if (width)
-          *width = data->window->width;
+          *width = data->window->config.bounds.w;
 
      if (height)
-          *height = data->window->height;
+          *height = data->window->config.bounds.h;
 
      return DFB_OK;
 }
@@ -302,7 +292,7 @@ static DFBResult
 IDirectFBWindow_GetSurface( IDirectFBWindow   *thiz,
                             IDirectFBSurface **surface )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -316,11 +306,11 @@ IDirectFBWindow_GetSurface( IDirectFBWindow   *thiz,
      if (!data->surface) {
           DFBResult ret;
 
-          DFB_ALLOCATE_INTERFACE( *surface, IDirectFBSurface );
+          DIRECT_ALLOCATE_INTERFACE( *surface, IDirectFBSurface );
 
           ret = IDirectFBSurface_Window_Construct( *surface,
                                                    NULL, NULL, data->window,
-                                                   DSCAPS_FLIPPING );
+                                                   DSCAPS_DOUBLE );
           if (ret)
                return ret;
 
@@ -338,7 +328,7 @@ static DFBResult
 IDirectFBWindow_SetOptions( IDirectFBWindow  *thiz,
                             DFBWindowOptions  options )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      /* Check arguments */
      if (data->destroyed)
@@ -351,24 +341,22 @@ IDirectFBWindow_SetOptions( IDirectFBWindow  *thiz,
           options &= ~DWOP_ALPHACHANNEL;
 
      /* Set new options */
-     data->window->options = options;
-
-     return DFB_OK;
+     return dfb_window_change_options( data->window, DWET_ALL, options );
 }
 
 static DFBResult
 IDirectFBWindow_GetOptions( IDirectFBWindow  *thiz,
-                            DFBWindowOptions *options )
+                            DFBWindowOptions *ret_options )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     if (!options)
+     if (!ret_options)
           return DFB_INVARG;
 
-     *options = data->window->options;
+     *ret_options = data->window->config.options;
 
      return DFB_OK;
 }
@@ -382,7 +370,7 @@ IDirectFBWindow_SetColorKey( IDirectFBWindow *thiz,
      __u32        key;
      CoreSurface *surface;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -397,14 +385,7 @@ IDirectFBWindow_SetColorKey( IDirectFBWindow *thiz,
      else
           key = dfb_color_to_pixel( surface->format, r, g, b );
 
-     if (data->window->color_key != key) {
-          data->window->color_key = key;
-
-          if (data->window->options & DWOP_COLORKEYING)
-               dfb_window_repaint( data->window, NULL, 0, false, false );
-     }
-
-     return DFB_OK;
+     return dfb_window_set_colorkey( data->window, key );
 }
 
 static DFBResult
@@ -413,7 +394,7 @@ IDirectFBWindow_SetColorKeyIndex( IDirectFBWindow *thiz,
 {
      __u32 key = index;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -421,14 +402,7 @@ IDirectFBWindow_SetColorKeyIndex( IDirectFBWindow *thiz,
      if (data->window->caps & DWCAPS_INPUTONLY)
           return DFB_UNSUPPORTED;
 
-     if (data->window->color_key != key) {
-          data->window->color_key = key;
-
-          if (data->window->options & DWOP_COLORKEYING)
-               dfb_window_repaint( data->window, NULL, 0, false, false );
-     }
-
-     return DFB_OK;
+     return dfb_window_set_colorkey( data->window, key );
 }
 
 static DFBResult
@@ -438,9 +412,9 @@ IDirectFBWindow_SetOpaqueRegion( IDirectFBWindow *thiz,
                                  int              x2,
                                  int              y2 )
 {
-     CoreWindow *window;
+     DFBRegion region;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -448,36 +422,28 @@ IDirectFBWindow_SetOpaqueRegion( IDirectFBWindow *thiz,
      if (x1 > x2 || y1 > y2)
           return DFB_INVAREA;
 
-     window = data->window;
-     
-     window->opaque.x1 = x1;
-     window->opaque.y1 = y1;
-     window->opaque.x2 = x2;
-     window->opaque.y2 = y2;
+     region = (DFBRegion) { x1, y1, x2, y2 };
 
-     return DFB_OK;
+     return dfb_window_set_opaque( data->window, &region );
 }
 
 static DFBResult
 IDirectFBWindow_SetOpacity( IDirectFBWindow *thiz,
                             __u8             opacity )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     if (data->window->opacity != opacity)
-          dfb_window_set_opacity( data->window, opacity );
-
-     return DFB_OK;
+     return dfb_window_set_opacity( data->window, opacity );
 }
 
 static DFBResult
 IDirectFBWindow_GetOpacity( IDirectFBWindow *thiz,
                             __u8            *opacity )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -485,7 +451,7 @@ IDirectFBWindow_GetOpacity( IDirectFBWindow *thiz,
      if (!opacity)
           return DFB_INVARG;
 
-     *opacity = data->window->opacity;
+     *opacity = data->window->config.opacity;
 
      return DFB_OK;
 }
@@ -498,7 +464,7 @@ IDirectFBWindow_SetCursorShape( IDirectFBWindow  *thiz,
 {
      DFBResult ret;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -511,7 +477,7 @@ IDirectFBWindow_SetCursorShape( IDirectFBWindow  *thiz,
      if (shape) {
           IDirectFBSurface_data *shape_data;
           CoreSurface           *shape_surface;
-          
+
           shape_data = (IDirectFBSurface_data*) shape->priv;
           if (!shape_data)
                return DFB_DEAD;
@@ -529,8 +495,8 @@ IDirectFBWindow_SetCursorShape( IDirectFBWindow  *thiz,
           data->cursor.hot_y = hot_y;
 
           if (data->entered)
-               dfb_layer_cursor_set_shape( data->layer,
-                                           shape_surface, hot_x, hot_y );
+               return dfb_windowstack_cursor_set_shape( data->window->stack,
+                                                        shape_surface, hot_x, hot_y );
      }
 
      return DFB_OK;
@@ -541,25 +507,26 @@ IDirectFBWindow_RequestFocus( IDirectFBWindow *thiz )
 {
      CoreWindow *window;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
      window = data->window;
-     
-     if ((window->options & DWOP_GHOST) || !window->opacity)
+
+     if (window->config.options & DWOP_GHOST)
           return DFB_UNSUPPORTED;
 
-     dfb_window_request_focus( window );
+     if (!window->config.opacity && !(window->caps & DWCAPS_INPUTONLY))
+          return DFB_UNSUPPORTED;
 
-     return DFB_OK;
+     return dfb_window_request_focus( window );
 }
 
 static DFBResult
 IDirectFBWindow_GrabKeyboard( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -570,7 +537,7 @@ IDirectFBWindow_GrabKeyboard( IDirectFBWindow *thiz )
 static DFBResult
 IDirectFBWindow_UngrabKeyboard( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -581,7 +548,7 @@ IDirectFBWindow_UngrabKeyboard( IDirectFBWindow *thiz )
 static DFBResult
 IDirectFBWindow_GrabPointer( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -592,7 +559,7 @@ IDirectFBWindow_GrabPointer( IDirectFBWindow *thiz )
 static DFBResult
 IDirectFBWindow_UngrabPointer( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -605,7 +572,7 @@ IDirectFBWindow_GrabKey( IDirectFBWindow            *thiz,
                          DFBInputDeviceKeySymbol     symbol,
                          DFBInputDeviceModifierMask  modifiers )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -618,7 +585,7 @@ IDirectFBWindow_UngrabKey( IDirectFBWindow            *thiz,
                            DFBInputDeviceKeySymbol     symbol,
                            DFBInputDeviceModifierMask  modifiers )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -629,7 +596,7 @@ IDirectFBWindow_UngrabKey( IDirectFBWindow            *thiz,
 static DFBResult
 IDirectFBWindow_Move( IDirectFBWindow *thiz, int dx, int dy )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -637,26 +604,18 @@ IDirectFBWindow_Move( IDirectFBWindow *thiz, int dx, int dy )
      if (dx == 0  &&  dy == 0)
           return DFB_OK;
 
-     dfb_window_move( data->window, dx, dy );
-
-     return DFB_OK;
+     return dfb_window_move( data->window, dx, dy, true );
 }
 
 static DFBResult
 IDirectFBWindow_MoveTo( IDirectFBWindow *thiz, int x, int y )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     if (data->window->x == x  &&  data->window->y == y)
-          return DFB_OK;
-
-     dfb_window_move( data->window,
-                      x - data->window->x, y - data->window->y );
-
-     return DFB_OK;
+     return dfb_window_move( data->window, x, y, false );
 }
 
 static DFBResult
@@ -664,16 +623,13 @@ IDirectFBWindow_Resize( IDirectFBWindow *thiz,
                         int              width,
                         int              height )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
      if (width < 1 || width > 4096 || height < 1 || height > 4096)
           return DFB_INVARG;
-     
-     if (data->window->width == width  &&  data->window->height == height)
-          return DFB_OK;
 
      return dfb_window_resize( data->window, width, height );
 }
@@ -681,21 +637,19 @@ IDirectFBWindow_Resize( IDirectFBWindow *thiz,
 static DFBResult
 IDirectFBWindow_Raise( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     dfb_window_raise( data->window );
-
-     return DFB_OK;
+     return dfb_window_raise( data->window );
 }
 
 static DFBResult
 IDirectFBWindow_SetStackingClass( IDirectFBWindow        *thiz,
                                   DFBWindowStackingClass  stacking_class )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -709,48 +663,40 @@ IDirectFBWindow_SetStackingClass( IDirectFBWindow        *thiz,
                return DFB_INVARG;
      }
 
-     dfb_window_change_stacking( data->window, stacking_class );
-
-     return DFB_OK;
+     return dfb_window_change_stacking( data->window, stacking_class );
 }
 
 static DFBResult
 IDirectFBWindow_Lower( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     dfb_window_lower( data->window );
-
-     return DFB_OK;
+     return dfb_window_lower( data->window );
 }
 
 static DFBResult
 IDirectFBWindow_RaiseToTop( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     dfb_window_raisetotop( data->window );
-
-     return DFB_OK;
+     return dfb_window_raisetotop( data->window );
 }
 
 static DFBResult
 IDirectFBWindow_LowerToBottom( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     dfb_window_lowertobottom( data->window );
-
-     return DFB_OK;
+     return dfb_window_lowertobottom( data->window );
 }
 
 static DFBResult
@@ -759,7 +705,7 @@ IDirectFBWindow_PutAtop( IDirectFBWindow *thiz,
 {
      IDirectFBWindow_data *lower_data;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -774,9 +720,7 @@ IDirectFBWindow_PutAtop( IDirectFBWindow *thiz,
      if (!lower_data->window)
           return DFB_DESTROYED;
 
-     dfb_window_putatop( data->window, lower_data->window );
-
-     return DFB_OK;
+     return dfb_window_putatop( data->window, lower_data->window );
 }
 
 static DFBResult
@@ -785,7 +729,7 @@ IDirectFBWindow_PutBelow( IDirectFBWindow *thiz,
 {
      IDirectFBWindow_data *upper_data;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -800,9 +744,7 @@ IDirectFBWindow_PutBelow( IDirectFBWindow *thiz,
      if (!upper_data->window)
           return DFB_DESTROYED;
 
-     dfb_window_putbelow( data->window, upper_data->window );
-
-     return DFB_OK;
+     return dfb_window_putbelow( data->window, upper_data->window );
 }
 
 static DFBResult
@@ -810,7 +752,7 @@ IDirectFBWindow_Close( IDirectFBWindow *thiz )
 {
      DFBWindowEvent evt;
 
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -825,13 +767,13 @@ IDirectFBWindow_Close( IDirectFBWindow *thiz )
 static DFBResult
 IDirectFBWindow_Destroy( IDirectFBWindow *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBWindow)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     DEBUGMSG("IDirectFBWindow_Destroy\n");
-     
+     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_Destroy()\n" );
+
      dfb_window_destroy( data->window );
 
      return DFB_OK;
@@ -840,27 +782,18 @@ IDirectFBWindow_Destroy( IDirectFBWindow *thiz )
 DFBResult
 IDirectFBWindow_Construct( IDirectFBWindow *thiz,
                            CoreWindow      *window,
-                           bool             init )
+                           CoreLayer       *layer )
 {
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBWindow)
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBWindow)
 
-     DEBUGMSG( "IDirectFBWindow_Construct: window at %d %d, size %dx%d\n",
-                window->x, window->y, window->width, window->height );
+     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_Construct() <- %d, %d - %dx%d\n",
+                 DFB_RECTANGLE_VALS( &window->config.bounds ) );
 
-     data->ref = 1;
+     data->ref    = 1;
      data->window = window;
-     data->layer  = dfb_layer_at( window->stack->layer_id );
+     data->layer  = layer;
 
-     dfb_window_attach( window, IDirectFBWindow_React,
-                        data, &data->reaction );
-
-     if (init) {
-          DFBWindowEventType old_events = window->events;
-
-          window->events |= DWET_POSITION_SIZE;
-          dfb_window_init( window );
-          window->events = old_events;
-     }
+     dfb_window_attach( window, IDirectFBWindow_React, data, &data->reaction );
 
      thiz->AddRef = IDirectFBWindow_AddRef;
      thiz->Release = IDirectFBWindow_Release;
@@ -913,28 +846,20 @@ IDirectFBWindow_React( const void *msg_data,
      const DFBWindowEvent *evt  = msg_data;
      IDirectFBWindow_data *data = ctx;
 
-     HEAVYDEBUGMSG("IDirectFBWindow_React\n");
+     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_React()\n");
 
      switch (evt->type) {
-          case DWET_POSITION_SIZE:
-               if (!data->position_size_event)
-                    data->position_size_event = DFBMALLOC( sizeof(DFBWindowEvent) );
-
-               *data->position_size_event = *evt;
-
-               break;
-
           case DWET_DESTROYED:
-               DEBUGMSG("IDirectFBWindow_React - window destroyed\n");
+               D_DEBUG_AT( IDirectFB_Window, "  -> window destroyed\n" );
 
                if (!data->destroyed) {
                     data->destroyed = true;
 
-                    DEBUGMSG("IDirectFBWindow_React - unrefing...\n");
+                    D_DEBUG_AT( IDirectFB_Window, "  -> unrefing...\n" );
 
                     dfb_window_unref( data->window );
 
-                    DEBUGMSG("IDirectFBWindow_React - unref done.\n");
+                    D_DEBUG_AT( IDirectFB_Window, "  -> unref done.\n" );
                }
 
                data->detached = true;
@@ -958,10 +883,10 @@ IDirectFBWindow_React( const void *msg_data,
                     if (!shape_data->surface)
                          break;
 
-                    dfb_layer_cursor_set_shape( data->layer,
-                                                shape_data->surface,
-                                                data->cursor.hot_x,
-                                                data->cursor.hot_y );
+                    dfb_windowstack_cursor_set_shape( data->window->stack,
+                                                      shape_data->surface,
+                                                      data->cursor.hot_x,
+                                                      data->cursor.hot_y );
                }
 
                break;
